#include "AgeClassHandler.h"
#include "Tree.h"
#include "classify.h"
#include "TreeContainer.h"
#include "Log.h"

// Xerces-C++ includes.
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMElement.hpp>
#include <xercesc/dom/DOMNodeList.hpp>
XERCES_CPP_NAMESPACE_USE

#include "XercesString.h"
#include "XercesUtilities.h"

#include <iterator>
#include <map>
using std::map;

AgeClassHandler::AgeClassHandler()
{
}

AgeClassHandler::~AgeClassHandler()
{
}

unsigned int AgeClassHandler::addAgeClass(string const & name, int minimumAge)
{
	ageClassNames.push_back(name);
	ageClassBounds.push_back(minimumAge);
	return getNumberOfAgeClasses() - 1;
}

unsigned int AgeClassHandler::determineAgeClass(Tree const & curTree) const
{
	if (ageClassBounds.empty()) {
		return 0;
	}
	int temp = classify(ageClassBounds, static_cast<int>(curTree.getAge()));
	unsigned int ageClass = static_cast<unsigned int>(temp > 0? temp: 0);
	return ageClass;
}

unsigned int AgeClassHandler::getAgeClassId(string const & name) const
{
	vector<string>::const_iterator position = find(ageClassNames.begin(), ageClassNames.end(), name);
	if (position != ageClassNames.end()) {
		return distance(ageClassNames.begin(), position);
	} else {
		throw "Age class not found.";
	}
}

void AgeClassHandler::loadData(DOMElement const * ageClasses)
{
	// eUnrecognized denotes an unrecognized tag.
	enum Tags { eUnrecognized, eAgeClass };

	map<string, int> tagMap;
	tagMap["ageClass"] = eAgeClass;

	for (DOMElement * element = XercesUtilities::getFirstChildDOMElement(ageClasses); element != 0; element = XercesUtilities::getNextSiblingDOMElement(element)) {
		XercesString tagName = XMLString::replicate(element->getTagName());
		switch (tagMap[tagName.toString()])
		{
			case (eAgeClass):
			{
				Log::log(2, "Age class:");
				loadAgeClass(element);
				break;
			}
			default:	// Handles eUnrecognized.
			{
				throw "Unrecognized XML-element: " + tagName.toString();
			}
		}
	}
}

void AgeClassHandler::loadAgeClass(DOMElement const * ageClass)
{
	string name;
	unsigned short minimumAge = 0;

	// eUnrecognized denotes an unrecognized tag.
	enum Tags { eUnrecognized, eName, eMinimumAge };

	map<string, int> tagMap;
	tagMap["name"] = eName;
	tagMap["minimumAge"] = eMinimumAge;

	for (DOMElement * element = XercesUtilities::getFirstChildDOMElement(ageClass); element != 0; element = XercesUtilities::getNextSiblingDOMElement(element)) {
		XercesString tagName = XMLString::replicate(element->getTagName());
		switch (tagMap[tagName.toString()])
		{
			case (eName):
			{
				name = XercesUtilities::getFirstChildTextContentAsString(element);
				Log::log(3, "Name: %s", name.c_str());
				break;
			}
			case (eMinimumAge):
			{
				minimumAge = XercesUtilities::getFirstChildTextContentAsInt(element);
				Log::log(3, "Minimum age: %d", minimumAge);
				break;
			}
			default:	// Handles eUnrecognized.
			{
				throw "Unrecognized XML-element: " + tagName.toString();
			}
		}
	}

	addAgeClass(name, minimumAge);
}
