#ifndef ASSORTMENTVOLUMEMODEL_H_
#define ASSORTMENTVOLUMEMODEL_H_

#include <vector>
using std::vector;

class Tree;
class Compartment;

class AssortmentVolumeModel {
public:
	enum { ALL_TREES, MARKED_TREES } IncludeTrees;

	explicit AssortmentVolumeModel(Compartment const & comp);

	/**
	 * Calculate the volume of each assortment type for a single tree.
	 * The result is a vector of volumes in cubic metres where the vector index
	 * is the index of the corresponding assortment as available in AssortmentContainer.
	 * The last element in the result vector (with an index one greater than the
	 * number of assortments) is the total volume of the tree.
	 */
	void calculateForTree(Tree const & tree, vector<double> & result);

	/**
	 * Calculate the volume of each assortment type for the entire compartment.
	 */
	void calculateForCompartment(vector<double> & result, int toInclude = ALL_TREES);

	/**
	 * Calculate the volume of each assortment type for the entire compartment
	 * and perform the calculation separately for each species. The first index
	 * of the result vector is the species ID.
	 */
	void calculateForCompartment(vector< vector<double> > & result, int toInclude = ALL_TREES);
private:
	Compartment const * compartment;
};

#endif /* ASSORTMENTVOLUMEMODEL_H_ */
