/*
 * CompetitionDistanceModel.cpp
 *
 *  Created on: 2008-okt-29
 *      Author: joschori
 */

#include "CompetitionDistanceModel.h"
#include "Tree.h"
#include "Species.h"
#include "SpeciesContainer.h"
#include "Model.h"
#include "SecondaryModel.h"
#include "Vector.h"
#include "SizeClassHandler.h"
#include "AgeClassHandler.h"
#include "Compartment.h"

#ifdef CALCULATION_INSPECTION
#include "CalculationInspector.h"
#endif

CompetitionDistanceModel::CompetitionDistanceModel(SizeClassHandler const & sizeClassHandler,
		AgeClassHandler const & ageClassHandler,
		HeightClassHandler const & heightClassHandler)
: sizeClassHandler(sizeClassHandler)
, ageClassHandler(ageClassHandler)
, heightClassHandler(heightClassHandler)
, secondaryVariables(SecondaryModel::numberOfVariables())
, modelVariables(Model::NUMBER_OF_MODEL_VARIABLES) {

}

CompetitionDistanceModel::~CompetitionDistanceModel() {

}

double CompetitionDistanceModel::calculate(Tree const * const curTree) {
	Species const * const species = SpeciesContainer::get(curTree->getSpeciesId());
	CompetitionDistanceData const & competitionDistanceData = species->getCompetitionDistanceData();

	SecondaryModel::calculateSecondaryVariables(
			curTree, curTree->getParentCompartment()->getCompartmentData(),
			sizeClassHandler, ageClassHandler, heightClassHandler,
			secondaryVariables);

	modelVariables.clear();
	modelVariables[Model::TAU] = curTree->getInitialHeterogeneity();
	// No environmental effects are considered
	modelVariables[Model::XI] = 0.0;

	// The competition distance is considered not to depend on neighbouring trees.
	// Hence it can be calculated with competition radius = 0.
	Model::calculateModelVariables(curTree, vector<Tree*>(), 0.0, modelVariables);
	double const competitionDistanceThisTree = Model::calculateMark(
			modelVariables, secondaryVariables, competitionDistanceData.getModelTerms());

#ifdef CALCULATION_INSPECTION
	CalculationInspector& inspector = CalculationInspector::Instance();
	if (inspector.shouldInspectCompetitionDistance()) {
		inspector.inspectModel("Competition Distance", *curTree, -1, -1, competitionDistanceData, modelVariables, secondaryVariables, competitionDistanceThisTree);
	}
#endif

	return competitionDistanceThisTree;
}
