#include "DiameterModel.h"
#include "Tree.h"
#include "SpeciesContainer.h"
#include "Species.h"

double DiameterModel::calculateDiameterAtHeight(
		Tree const & tree, double position, double diameterAt20PercentOfHeight) {
	double const height = tree.getMark(TreeMarks::HEIGHT);
	if (position > height)
		return 0;

	Species const * const species = SpeciesContainer::get(tree.getSpeciesId());
	if (diameterAt20PercentOfHeight == 0.0) {
		// Diameter at 20 percent of height not given. Estimate it.
		diameterAt20PercentOfHeight = estimateDiameterAt20PercentOfHeight(tree);
	}
	double const x = 1.0 - position / height;
	return diameterAt20PercentOfHeight * calculateRelativeDiameter(*species, x);
}

double DiameterModel::estimateDiameterAt20PercentOfHeight(Tree const & tree) {
	Species const * const species = SpeciesContainer::get(tree.getSpeciesId());
	double const dbh = tree.getMark(TreeMarks::DIAMETER);
	double const height = tree.getMark(TreeMarks::HEIGHT);
	double const x_dbh = 1.0 - 1.3 / height;
	return dbh / calculateRelativeDiameter(*species, x_dbh);
}

double DiameterModel::calculateRelativeDiameter(Species const & species, double const x) {
	double xa = x;
	double xb = x * x;
	double result = species.get(Species::TAPER_COEFFICIENT1) * xa;
	result += species.get(Species::TAPER_COEFFICIENT2) * xb;
	xa *= xb;
	result += species.get(Species::TAPER_COEFFICIENT3) * xa;
	xb *= xa;
	result += species.get(Species::TAPER_COEFFICIENT5) * xb;
	xa *= xb;
	result += species.get(Species::TAPER_COEFFICIENT8) * xa;
	xb *= xa;
	result += species.get(Species::TAPER_COEFFICIENT13) * xb;
	xa *= xb;
	result += species.get(Species::TAPER_COEFFICIENT21) * xa;
	xb *= xa;
	result += species.get(Species::TAPER_COEFFICIENT34) * xb;
	return result;
}

