#ifndef DIAMETERMODEL_H_
#define DIAMETERMODEL_H_

class Tree;
class Species;

/**
 * The DiameterModel class contains static methods for estimating the diameter
 * of a tree at a specific height.
 */
class DiameterModel {
public:
	/**
	 * Calculate the diameter of a tree at a specific height.
	 * @param tree The tree
	 * @param position The height position (from zero to the tree's height)
	 * @param diameterAt20PercentOfHeight If zero is given, the
	 * estimateDiameterAt20PercentOfHeight() method is used to estimate the value
	 */
	static double calculateDiameterAtHeight(
			Tree const & tree, double position,
			double diameterAt20PercentOfHeight = 0.0);

	/**
	 * Estimate the diameter at 20% of the tree's height.
	 */
	static double estimateDiameterAt20PercentOfHeight(Tree const & tree);
private:
	static double calculateRelativeDiameter(Species const & species, double const x);
};

#endif /* DIAMETERMODEL_H_ */
