#include "EvaluatorFunctions.h"
#include "Compartment.h"
#include "TreeContainer.h"
#include "Log.h"
#include "Tree.h"

#include <cmath>
using std::pow;

double EvaluatorFunctions::calculateBasalAreaSum(Compartment const * const compartment) {
	double sumBasalArea = 0.0;
	double const fromDiameterInCentimeterToRadiusInMeterCoefficient = 1.0/200.0;
	for(Tree * curTree = compartment->getFirstTree(); curTree!=NULL; curTree=curTree->getNextTreeInCompartment()) {
		double const radius = curTree->getMark(TreeMarks::DIAMETER)*fromDiameterInCentimeterToRadiusInMeterCoefficient;
		sumBasalArea += M_PI * (radius*radius); // area = pi * r^2
	}
	return sumBasalArea;
}
double EvaluatorFunctions::calculateMarkSum(Compartment const * const compartment, unsigned int const mark) {
	double sum = 0.0;
	for(Tree * curTree = compartment->getFirstTree(); curTree!=NULL; curTree=curTree->getNextTreeInCompartment()) {
		sum += curTree->getMark(mark);
	}
	return sum;
}
void EvaluatorFunctions::calculateSpecies(TreeContainer const & trees, vector< map<size_t, uint64_t > > & speciesSums) {
	for(TreeContainer::const_iterator treeIter=trees.begin(); treeIter!=trees.end(); ++treeIter) {
		Tree const * const curTree = *treeIter;
		unsigned int const speciesId = curTree->getSpeciesId();
		unsigned int const compartmentId = curTree->getParentCompartment()->getCompartmentData()->getIdNumber();
		speciesSums[speciesId][compartmentId]++;
	}
}
double EvaluatorFunctions::calculateMean(double const sum, uint64_t const size) {
	if(size==0)
		return 0.0;
	else
		return sum/size;
}
double EvaluatorFunctions::safeDivision(double const sum, double const divisor) {
	if(divisor==0.0)
		return 0.0;
	else
		return sum/divisor;
}
double EvaluatorFunctions::checkingPow(double const base, double const exponent) {
	if ((base < 0.0) && (exponent != (double) ((int) exponent)))
		throw "Negative base and non-integer exponent is not allowed in base^exponent!";
	if ((base == 0.0) && (exponent < 0.0))
		throw "Base equal to zero and negative exponent is not allowed in base^exponent!";
	return pow(base, exponent);
}
