#ifndef FOREST_H_
#define FOREST_H_

#include <vector>
using std::vector;

#include "TreeContainer.h"

class CompartmentDataContainer;
class CompartmentData;
class SimulationData;

class Forest
{
public:
	Forest(CompartmentDataContainer const & dataCont, SimulationData const & simulationData, int rep);
	~Forest();

	/*
	 * Simulates the behaviour of the forest. All iterations.
	 * One iteration consists of growth, mortality and reproduction.
	 */
	void simulate();
	inline TreeContainer & getTreeContainer() { return allTrees; }
private:

#ifdef USE_MPI
	void transferForeignTrees();
	void clearForeignTrees();
#endif

	void createInitialForest();
	/*
	 * Simulates the behaviour of the forest during one iteration (time step)
	 * One iteration consists of growth, mortality and reproduction.
	 */
	void simulateIteration(unsigned int iteration);
	/**
	 * calculateGrowthStatistics() calculates and stores basal area after growth and volume after growth.
	 */
	void calculateGrowthStatistics(unsigned int iteration);
	/**
	 * calculateAssortmentStatistics() calculates the volume of assortment types.
	 */
	void calculateAssortmentStatistics(unsigned int iteration);
	/**
	 * calculateRemovalAssortmentStatistics() calculates the volume of assortment types of removed trees.
	 */
	void calculateRemovalAssortmentStatistics(unsigned int iteration);
	/**
	 * calculateCompartmentCharacteristics() calculates basal area, volume and diameter sums.
	 * contains mpi communication and will force processes to sync.
	 */
	void calculateCompartmentCharacteristics();

	void calculateGrowth(unsigned int iteration);
	unsigned int calculateMortality(unsigned int iteration);
	void calculateManagement(unsigned int iteration);
	void calculateReproduction(unsigned int iteration);
	void calculateNewTreeMarks(TreeContainer & newTrees, unsigned int iteration);

	void addGrowthTreeStatistics(unsigned int iteration);
	void addRemovalStatistics(unsigned int iteration);
	void addFinalTreeStatistics(unsigned int iteration);
	void growAndAgeTrees();
	void addNewTrees(unsigned int iteration);
	void storeAgeStatistics();

	TreeContainer					allTrees;
// Only used when using COMPARISONSEEDING
	TreeContainer					otherCompartmentTrees;
	CompartmentDataContainer const &	dataContainer;
	SimulationData const &			simulationData;
	int								replication;
};

#endif /*FOREST_H_*/
