#include "HeightClassHandler.h"
#include "Tree.h"
#include "classify.h"
#include "TreeContainer.h"
#include "Log.h"

// Xerces-C++ includes.
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMElement.hpp>
#include <xercesc/dom/DOMNodeList.hpp>
XERCES_CPP_NAMESPACE_USE

#include "XercesString.h"
#include "XercesUtilities.h"

#include <iterator>
#include <map>
using std::map;

HeightClassHandler::HeightClassHandler()
{
}

HeightClassHandler::~HeightClassHandler()
{
}

unsigned int HeightClassHandler::addHeightClass(string const & name, double minimumHeight)
{
	heightClassNames.push_back(name);
	heightClassBounds.push_back(minimumHeight);
	return getNumberOfHeightClasses() - 1;
}

unsigned int HeightClassHandler::determineHeightClass(Tree const & curTree) const
{
	if (heightClassBounds.empty()) {
		return 0;
	}
	int temp = classify(heightClassBounds, curTree.getMark(TreeMarks::HEIGHT));
	unsigned int heightClass = static_cast<unsigned int>(temp > 0? temp: 0);
	return heightClass;
}

unsigned char HeightClassHandler::getHeightClassId(string const & name) const
{
	vector<string>::const_iterator position = find(heightClassNames.begin(), heightClassNames.end(), name);
	if(position != heightClassNames.end()) {
		return distance(heightClassNames.begin(), position);
	} else {
		throw "Size class not found.";
	}
}

void HeightClassHandler::loadData(DOMElement const * heightClasses)
{
	// eUnrecognized denotes an unrecognized tag.
	enum Tags { eUnrecognized, eHeightClass };

	map<string, int> tagMap;
	tagMap["heightClass"] = eHeightClass;

	for (DOMElement * element = XercesUtilities::getFirstChildDOMElement(heightClasses); element != 0; element = XercesUtilities::getNextSiblingDOMElement(element)) {
		XercesString tagName = XMLString::replicate(element->getTagName());
		switch (tagMap[tagName.toString()])
		{
			case (eHeightClass):
			{
				Log::log(2, "Height class:");
				loadHeightClass(element);
				break;
			}
			default:	// Handles eUnrecognized.
			{
				throw "Unrecognized XML-element: " + tagName.toString();
			}
		}
	}
}

void HeightClassHandler::loadHeightClass(DOMElement const * heightClass)
{
	string name;
	double minimumHeight = 0.0;

	// eUnrecognized denotes an unrecognized tag.
	enum Tags { eUnrecognized, eName, eMinimumHeight };

	map<string, int> tagMap;
	tagMap["name"] = eName;
	tagMap["minimumHeight"] = eMinimumHeight;

	for (DOMElement * element = XercesUtilities::getFirstChildDOMElement(heightClass); element != 0; element = XercesUtilities::getNextSiblingDOMElement(element)) {
		XercesString tagName = XMLString::replicate(element->getTagName());
		switch (tagMap[tagName.toString()])
		{
			case (eName):
			{
				name = XercesUtilities::getFirstChildTextContentAsString(element);
				Log::log(3, "Name: %s", name.c_str());
				break;
			}
			case (eMinimumHeight):
			{
				minimumHeight = XercesUtilities::getFirstChildTextContentAsDouble(element);
				Log::log(3, "Minimum height: %f", minimumHeight);
				break;
			}
			default:	// Handles eUnrecognized.
			{
				throw "Unrecognized XML-element: " + tagName.toString();
			}
		}
	}

	addHeightClass(name, minimumHeight);
}
