#ifndef MATRIX_H_
#define MATRIX_H_
#include "acml.h"

#include <ostream>
using std::ostream;
#include <vector>
using std::vector;

#include "Vector.h"

/**
 * A matrix class with support for basic matrix operations.
 */
class Matrix
{
public:
	Matrix();
	Matrix(size_t nrOfRows, size_t nrOfColumns, double defaultValue=0.0);
	Matrix(Matrix const & other); // Copy constructor with deep copy
	~Matrix();

	enum OrientationType { UPPER_TRIANGULAR, LOWER_TRIANGULAR };

	inline double* operator[](size_t rowIndex) const {
		// Return a pointer to the requested row.
		return array + (rowIndex*numberOfColumns);
	}
	inline double * getDataPointer() const {
		return array;
	}

	inline Matrix & operator=(Matrix const & other) { // Assignment operator with deep copy
		if(numberOfRows!=other.numberOfRows || numberOfColumns!=other.numberOfColumns) {
			delete[] array;
			numberOfRows=other.numberOfRows;
			numberOfColumns=other.numberOfColumns;
			array = new double[numberOfRows*numberOfColumns];
			capacity = numberOfRows*numberOfColumns;
		}
		copyMatrix(other,*this);
		return *this;
	}

	inline void clear() {
		size_t const size = getNumberOfRows()*getNumberOfColumns();
		for(size_t i=0; i < size; ++i) {
			array[i]=0.0;
		}
	}

	void resize(size_t nrOfRows, size_t nrOfColumns);

	/* The Vector created and return here does in fact only contain a reference
	 * to a row in this Matrix so no data is copied or allocated.
	 */
	inline Vector getRow(size_t row) const {
		return Vector(array + row*numberOfColumns, numberOfColumns);
	}
	inline size_t getNumberOfRows() const {
		return numberOfRows;
	}
	inline size_t getNumberOfColumns() const {
		return numberOfColumns;
	}
	/**
	 * Matrix-vector multiplication.
	 */
	Vector multiplyWith(Vector const & vec) const;
	Vector transposedMultiplyWith(Vector const & vec) const;
	/**
	 * Cholesky decomposition (factorization) applied to matrix
	 */
	void choleskyDecompose(OrientationType const type);
	friend ostream & operator<<(ostream & os, Matrix const & m);
private:

	inline static void copyMatrix(Matrix const & from, Matrix & to) {
		size_t const size = to.getNumberOfColumns()*to.getNumberOfRows();
		dcopy(size, from.getDataPointer(), 1, to.getDataPointer(), 1);
	}

	double *		array;
	size_t			numberOfColumns;
	size_t			numberOfRows;
	size_t			capacity;
};

ostream & operator<<(ostream & os, Matrix const & m);

#endif /*MATRIX_H_*/
