#ifndef MODEL_H_
#define MODEL_H_

#include <string>
using std::string;
#include <vector>
using std::vector;

class Tree;
class Vector;
class ModelFactor;
class ModelTerm;

/**
 * Model provides functions needed in the initialization, growth, mortality and reproduction processes.
 * It also handles the mapping from competition index names to ids.
 */
class Model
{
public:
	enum CompetitionIndexes { INTERCEPT = 0, DIAMETER, DIAMETER_LOG, DENSITY, AVG_DIST, INV_DIST, DIAMETER_SUM, DIAMETER_DIFF, DIAMETER_RATIO, BA_DIAMETER_RATIO, TAU, XI, EPSILON, HEIGHT_DIFF, HEIGHT_DIAMETER_RATIO, COMPETITOR_BASAL_AREA, RELATIVE_BASAL_AREA, COMPARTMENT_VOLUME, NUMBER_OF_MODEL_VARIABLES };

	/**
	 * Returns the name of the model variable corresponding to the specified id.
	 */
	static string getModelVariableName(unsigned int id);
	/**
	 * Returns the id of the model variable corresponding to the specified name.
	 */
	static unsigned int getModelVariableId(string const & name);
	/**
	 * Evaluates the mixed linear model using the specified parameters as input.
	 */
	static double calculateMark(Vector & modelVariables, Vector & secondaryVariables, vector<ModelTerm> const & modelTerms);
	/**
	 * Calculates model variables for the specified target tree.
	 * The results are stored in @modelVariables.
	 */
	static void calculateModelVariables(Tree const * const targetTree, vector<Tree*> const & trees, float const competitionRadius, Vector & modelVariables);

private:
	static vector<string> setupModelVariableNames();

	static vector<string>	modelVariableNames;
};

#endif /*MODEL_H_*/
