/*
 * RegenerationModel.cpp
 *
 *  Created on: 2008-sep-10
 *      Author: joschori
 */

#include "RegenerationModel.h"
#include "Helper.h"
#include "EvaluatorFunctions.h"
#include "Compartment.h"
#include "Vector.h"
#include "StandardNormalRandomNumberGenerator.h"

vector<string> RegenerationModel::variableNames = setupVariableNames();

string RegenerationModel::getVariableName(unsigned int id) {
	return variableNames[id];
}

unsigned int RegenerationModel::getVariableId(string const & name) {
	return Helper::getVariableId(name,variableNames);
}

double RegenerationModel::calculateMark(Vector const & variables, Vector const & coefficients) {
	return Vector::dotProduct(variables, coefficients);
}

void RegenerationModel::calculateVariables(Compartment const * const compartment, Vector const & coefficients, Vector & variables) {
	variables.clear();
	variables[INTERCEPT] = 1.0;
	double const totalAreaInHectares = compartment->getPolygon()->getFullArea() * 0.0001;
	if(coefficients[COMPARTMENT_VOLUME]) {
		double const totalVolume =  compartment->getTotalVolume();
		variables[COMPARTMENT_VOLUME] = EvaluatorFunctions::safeDivision(totalVolume,totalAreaInHectares);
	}
	if(coefficients[COMPARTMENT_BASAL_AREA]) {
		double const totalBA = compartment->getTotalBasalArea();
		variables[COMPARTMENT_BASAL_AREA] = EvaluatorFunctions::safeDivision(totalBA,totalAreaInHectares);
	}
	if(coefficients[COMPARTMENT_TREES]) {
		uint64_t const totalNum = compartment->getTotalSize();
		variables[COMPARTMENT_TREES] = EvaluatorFunctions::safeDivision(totalNum,totalAreaInHectares);
	}
	if(coefficients[INV_VOLUME]) {
		double const totalVolume =  compartment->getTotalVolume();
		variables[INV_VOLUME] = EvaluatorFunctions::safeDivision(totalAreaInHectares,totalVolume);
	}
	if(coefficients[INV_BASAL_AREA]) {
		double const totalBA = compartment->getTotalBasalArea();
		variables[INV_BASAL_AREA] = EvaluatorFunctions::safeDivision(totalAreaInHectares, totalBA);
	}
	if(coefficients[INV_TREES]) {
		uint64_t const totalNum = compartment->getTotalSize();
		variables[INV_TREES] = EvaluatorFunctions::calculateMean(totalAreaInHectares,totalNum);
	}
	if(coefficients[EPSILON]) {
		variables[EPSILON] = StandardNormalRandomNumberGenerator::getLimited();
	}
}

vector<string> RegenerationModel::setupVariableNames() {
	vector<string> names;
	names.resize(NUMBER_OF_VARIABLES);
	names[INTERCEPT] = "Intercept";
	names[COMPARTMENT_VOLUME] = "Volume";
	names[COMPARTMENT_BASAL_AREA] = "Basal area";
	names[COMPARTMENT_TREES] = "Trees";
	names[INV_VOLUME] = "Inverse volume";
	names[INV_BASAL_AREA] = "Inverse basal area";
	names[INV_TREES] = "Inverse trees";
	names[EPSILON] = "Epsilon";
	return names;
}
