/*
 * RegenerationModel.h
 *
 *  Created on: 2008-sep-10
 *      Author: joschori
 */
#ifndef REGENERATIONMODEL_H_
#define REGENERATIONMODEL_H_

#include <string>
using std::string;
#include <vector>
using std::vector;

class Vector;
class Compartment;

class RegenerationModel {
public:
	enum Variables { INTERCEPT = 0, COMPARTMENT_VOLUME, COMPARTMENT_BASAL_AREA, COMPARTMENT_TREES, INV_VOLUME, INV_BASAL_AREA, INV_TREES, EPSILON, NUMBER_OF_VARIABLES };

	/**
	 * Returns the name of the model variable corresponding to the specified id.
	 */
	static string getVariableName(unsigned int id);
	/**
	 * Returns the id of the model variable corresponding to the specified name.
	 */
	static unsigned int getVariableId(string const & name);
	/**
	 * Evaluates the mixed linear model using the specified parameters as input.
	 */
	static double calculateMark(Vector const & variables, Vector const & coefficients);
	/**
	 * Calculates variables for the compartment.
	 * The results are stored in @modelVariables.
	 */
	static void calculateVariables(Compartment const * const compartment, Vector const & coefficients, Vector & variables);

private:
	static vector<string> setupVariableNames();

	static vector<string>	variableNames;

};

#endif /* REGENERATIONMODEL_H_ */
