#ifndef SECONDARYMODEL_H_
#define SECONDARYMODEL_H_

#include <string>
using std::string;
#include <vector>
using std::vector;

class Tree;
class CompartmentData;
class SizeClassHandler;
class AgeClassHandler;
class HeightClassHandler;
class Vector;
class SecondaryModelTerm;

/**
 * SecondaryModel provides functions needed to calculate the scaling factors for the competition indexes in Model.
 * It also handles the mapping from design parameter names to ids.
 */
class SecondaryModel
{
public:
	enum SecondaryVariables { INTERCEPT = 0, MANAGEMENT, SPECIES, AGE, AGE_CLASS, SIZE_CLASS, HEIGHT_CLASS, EPSILON, INDICATOR_VARIABLE };

	/**
	 * Must be called before SecondaryVariables can be used
	 * @param indicatorVariables  Names of species
	 */
	static void initialize(vector<string> const & indicatorVariables);

	/**
	 * Returns the number of secondary variables
	 */
	static size_t numberOfVariables();
	/**
	 * Returns the name of the secondary model variable corresponding to the specified id.
	 */
	static string getSecondaryVariableName(unsigned int id);
	/**
	 * Returns the id of the secondary model variable corresponding to the specified name.
	 */
	static unsigned int getSecondaryVariableId(string const & name);
	/**
	 * Evaluates the mixed linear model using the specified parameters as input.
	 */
	static double calculateMark(Vector & secondaryVariables, vector<SecondaryModelTerm> const & terms);
	/**
	 * Calculates secondary model variables for the specified target tree.
	 * The results are stored in @variables.
	 */
	static void calculateSecondaryVariables(
			Tree const * const targetTree,
			CompartmentData const * const compartmentData,
			SizeClassHandler const & sizeClassHandler,
			AgeClassHandler const & ageClassHandler,
			HeightClassHandler const & heightClassHandler,
			Vector & variables);
private:
	static vector<string> setupSecondaryVariableNames();

	static vector<string>	secondaryVariableNames;
};

#endif /*SECONDARYMODEL_H_*/
