#ifndef SIMULATION_H_
#define SIMULATION_H_

#include <ostream>
using std::ostream;
#include <string>
using std::string;

#include <xercesc/util/XercesVersion.hpp>
#include <xercesc/dom/DOMDocument.hpp>

#include "SimulationData.h"
#include "CompartmentDataContainer.h"

class StandStructureEvaluator;

namespace XERCES_CPP_NAMESPACE {
	class DOMElement;
}

class Simulation {
public:
	Simulation();
	~Simulation();

	void run();

	/**
	 * Loads input parameters in XML format from the file identified by @fileName.
	 */
	void loadInputParameters(string fileName);

	void createStatisticalOutput(StandStructureEvaluator & standStructureEvaluator);
	/**
	 * Prints a summary of the simulation to the specified ostream.
	 */
	void printSummary(unsigned int compartment, ostream & out) const;

private:
#ifdef USE_MPI
	void printNeighbours();
	void broadcastInputParameters();
#endif
	void printModels();
	/**
	 * Loads XML data for the Simulation.
	 *
	 * Precondition:	@simulation must not be null.
	 */
	void loadData(XERCES_CPP_NAMESPACE::DOMElement const * simulation);
	string loadOutputDirectory(XERCES_CPP_NAMESPACE::DOMDocument const * document);
	vector<string> loadIndicatorVariables(XERCES_CPP_NAMESPACE::DOMElement const * simulation);

	SimulationData simulationData;
	CompartmentDataContainer dataContainer;
};

#endif /*SIMULATION_H_*/
