#ifndef STANDSTATISTICSSUMMARY_H_
#define STANDSTATISTICSSUMMARY_H_

#include <vector>
using std::vector;
#include <ostream>
using std::ostream;
#include <string>
using std::string;

class SimulationData;
class CompartmentDataContainer;
class StandStructureEvaluator;
class Polygon;
class CompartmentData;

class StandStatisticsSummary {
public:
	struct PolygonCompartment {
		Polygon const * polygon;
		CompartmentData const * comp;
	};

	StandStatisticsSummary(
			SimulationData const & simData,
			CompartmentDataContainer const & compData,
			StandStructureEvaluator const & evaluator);
	~StandStatisticsSummary();

	void print(ostream & ostr) const;
private:
	vector<PolygonCompartment> generatePolygonCompartmentVector() const;

	double sumAndAverage(vector< vector<double> > const & statistics) const;

	double average(vector<double> const & statistics) const;

	void printHeader(ostream & ostr) const;

	void printAssortmentHeader(string const & prefix, ostream & ostr) const;

	void printPolygon(Polygon const * polygon, CompartmentData const * comp, ostream & ostr) const;

	void printRemovalAssortments(Polygon const * polygon, CompartmentData const * comp, ostream & ostr) const;

	void printAssortments(Polygon const * polygon, CompartmentData const * comp, ostream & ostr) const;

	SimulationData const & simData;
	CompartmentDataContainer const & compData;
	StandStructureEvaluator const & evaluator;
};

#endif

