#ifndef STANDSTRUCTUREEVALUATOR_H_
#define STANDSTRUCTUREEVALUATOR_H_

#include <ostream>
using std::ostream;
#include <string>
using std::string;
#include <vector>
using std::vector;
#include <map>
using std::map;

class TreeContainer;
class SimulationData;
class CompartmentStatsAccumulator;
class CompartmentDataContainer;

/**
 * StandStructureEvaluator calculates statistics.
 * These are now only mean DBH and basal area per hectare.
 */
class StandStructureEvaluator
{
public:
	StandStructureEvaluator(SimulationData const & simData, CompartmentDataContainer const & compData);
	~StandStructureEvaluator();

	/**
	 * Evaluates compartment structures for all compartments in a single replication.
	 */
	void evaluateCompartments(TreeContainer & trees, int const replication);
	/**
	 * Prints a summary of the compartment structure to the specified ostream.
	 */
	void printSummary(unsigned int compartment, ostream & out);
private:
	/**
	 * Calculates the sum of the simulated values and sorts the elements.
	 */
	void sumAndSort(vector<double> & simulatedValues, double & simulatedValuesSum);

	// Convenience functions for printing.
	void writeHeader(ostream & out, string firstHeader) const;
	void writeLine(ostream & out, int precision1, double lowerBound, double simulatedValuesSum, double upperBound) const;

	SimulationData const &			simulationData;
	CompartmentDataContainer const & compDataContainer;

	// Simulated compartment data.
	map<unsigned int,CompartmentStatsAccumulator> compartmentStats;

	friend class StandStatisticsSummary;
};

#endif /*STANDSTRUCTUREEVALUATOR_H_*/
