#include "TreeCharacteristicsVerifier.h"
#include "TreeContainer.h"
#include "Tree.h"
#include "TreeMarks.h"
#include "Log.h"
#include <math.h>

using namespace std;

#define ERROR_MSG_LIMIT 50

void TreeCharacteristicsVerifier::checkGrowth(TreeContainer const & trees) {
	unsigned int errors = 0;
	TreeContainer::const_iterator treeIter, endIter;
	for (treeIter = trees.begin(), endIter = trees.end(); treeIter != endIter; ++treeIter) {
		for (unsigned int mark = 0; mark < TreeMarks::NUMBER_OF_MARKS; mark++) {
			Tree const * const curTree = *treeIter;
			double const value = curTree->getGrowth(mark);
			if (value == 0.0 || isnan(value) || isinf(value)) {
				if (errors < ERROR_MSG_LIMIT) {
					Log::logWarning(4, "%s growth of tree at (%.4f, %.4f) is %.4f",
							TreeMarks::getMarkName(mark).c_str(),
							curTree->getX(), curTree->getY(), value);
				}
				errors++;
			}
		}
	}
	if (errors > ERROR_MSG_LIMIT) {
		Log::logWarning(-1, "%u other trees also have mark growth out of bounds", errors - ERROR_MSG_LIMIT);
	}
}

void TreeCharacteristicsVerifier::checkMarks(TreeContainer const & trees) {
	unsigned int errors = 0;
	TreeContainer::const_iterator treeIter, endIter;
	for (treeIter = trees.begin(), endIter = trees.end(); treeIter != endIter; ++treeIter) {
		for (unsigned int mark = 0; mark < TreeMarks::NUMBER_OF_MARKS; mark++) {
			Tree const * const curTree = *treeIter;
			double const value = curTree->getMark(mark);
			if (value < 0.0 || isnan(value) || isinf(value)) {
				if (errors < ERROR_MSG_LIMIT) {
					Log::logWarning(4, "%s of tree at (%.4f, %.4f) is %.4f",
							TreeMarks::getMarkName(mark).c_str(),
							curTree->getX(), curTree->getY(), value);
				}
				errors++;
			}
		}
	}
	if (errors > ERROR_MSG_LIMIT) {
		Log::logWarning(-1, "%u other trees also have mark values out of bounds", errors - ERROR_MSG_LIMIT);
	}
}

