#ifndef VOLUMEMODEL_H_
#define VOLUMEMODEL_H_

class Tree;
class Compartment;

/**
 * The VolumeModel calculates the volume of trees based on Laasasenaho's
 * taper curve functions.
 */
class VolumeModel {
public:
	static double calculateStemVolumeOf(Tree const * const targetTree);
	static double calculateStemVolumeOf(Tree const * const targetTree,
				double h0, double h1);
	static double calculateStemVolumeOf(Compartment const * const compartment);
	template<class T> static double calculateStemVolume(T const & trees);

	static double calculateBiomassVolumeOf(Tree const * const targetTree);
	static double calculateBiomassVolumeOf(Compartment const * const compartment);
	template<class T> static double calculateBiomassVolume(T const & trees);

private:
	static double gslIntegrand(double x, void * params);
};

template<class T> double VolumeModel::calculateStemVolume(T const & trees) {
	double volumeSum = 0.0;
	for(typename T::const_iterator treeIter=trees.begin(); treeIter!=trees.end(); ++treeIter) {
		volumeSum += calculateStemVolumeOf(*treeIter);
	}
	return volumeSum;
}

template<class T> double VolumeModel::calculateBiomassVolume(T const & trees) {
	return calculateStemVolume(trees) / 0.68;
}

#endif /*VOLUMEMODEL_H_*/
