#ifndef CLASSIFY_H_
#define CLASSIFY_H_

#include <algorithm>
using std::upper_bound;

/**
 * Classifies @value according to the bounds in @classBounds.
 * @classBounds must be in ascending order, otherwise classify() won't work.
 * 
 * classBounds:  [0]                [1]                [2]
 *    ------------|------------------|------------------|---------------
 * value:    <    |  <=           <  |  <=           <  |    <=
 *    ------------|------------------|------------------|---------------
 * class:   -1    |         0        |         1        |     2 
 * 
 */
template<class ForwardContainer, class T>
int classify(ForwardContainer const & classBounds, T const & value)
{
   typename ForwardContainer::const_iterator iter = upper_bound(classBounds.begin(), classBounds.end(), value);
   return distance(classBounds.begin(), iter) - 1;
}

#endif /*CLASSIFY_H_*/
