#ifndef ASSORTMENT_H_
#define ASSORTMENT_H_

#include <string>
using std::string;
#include <map>
using std::map;

#ifdef USE_MPI
#include "MPIUtil.h"
#endif

#include <xercesc/util/XercesVersion.hpp>

namespace XERCES_CPP_NAMESPACE
{
	class DOMElement;
}

class Assortment {
public:
	struct Constraints {
#ifdef USE_MPI
	inline void broadcast(int const rootId, MPI_Comm const comm) {
		MPIUtil::broadcast(minDiameter, rootId, comm);
		MPIUtil::broadcast(logLength, rootId, comm);
	}
#endif
		float minDiameter;
		float logLength;
	};

	typedef map<unsigned int, Constraints> ConstraintMap;

	Assortment();

#ifdef USE_MPI
	void broadcast(int const rootId, MPI_Comm const comm);
#endif
	void loadData(XERCES_CPP_NAMESPACE::DOMElement const * assortment);

	inline string const & getName() const {
		return name;
	}
	inline void setName(string const & name) {
		this->name = name;
	}

	inline Constraints const & getDefaultConstraints() const {
		return defaultConstraints;
	}
	inline Constraints & getDefaultConstraints() {
		return defaultConstraints;
	}

	inline Constraints const & getConstraints(unsigned int speciesId) const {
		ConstraintMap::const_iterator i = constraints.find(speciesId);
		if (i != constraints.end()) {
			return i->second;
		} else {
			return defaultConstraints;
		}
	}
	inline Constraints & getConstraints(unsigned int speciesId) {
		return constraints[speciesId];
	}
private:
	void loadConstraints(XERCES_CPP_NAMESPACE::DOMElement const * data, Constraints & result);

	string name;
	Constraints defaultConstraints;
	ConstraintMap constraints;
};

#endif /* ASSORTMENT_H_ */
