#include "AssortmentContainer.h"
#include "Log.h"

#include "XercesString.h"
#include "XercesUtilities.h"
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMElement.hpp>
XERCES_CPP_NAMESPACE_USE

#include <map>
using std::map;
#include <algorithm>
using std::stable_sort;
#include <cassert>

vector<Assortment> AssortmentContainer::assortments;

void AssortmentContainer::Destroy() {
	assortments.clear();
}

#ifdef USE_MPI
void AssortmentContainer::broadcast(int const rootId, MPI_Comm const comm) {
	unsigned int numAssortments = assortments.size();
	MPIUtil::broadcast(numAssortments, rootId, comm);
	if(MPIUtil::mpiId() != rootId)
		assortments.resize(numAssortments);
	for(unsigned int i = 0; i < numAssortments; ++i) {
		assortments[i].broadcast(rootId, comm);
	}
}
#endif

void AssortmentContainer::loadData(XERCES_CPP_NAMESPACE::DOMElement const * assortmentData) {
	// eUnrecognized denotes an unrecognized tag.
	enum Tags {eUnrecognized, eAssortment};

	map<string, int> tagMap;
	tagMap["assortment"] = eAssortment;

	for (DOMElement * element = XercesUtilities::getFirstChildDOMElement(assortmentData); element != 0; element
			= XercesUtilities::getNextSiblingDOMElement(element)) {
		XercesString tagName = XMLString::replicate(element->getTagName());
		switch (tagMap[tagName.toString()]) {
		case (eAssortment): {
			Assortment assortment;
			assortment.loadData(element);
			string str = "Loaded assortment: " + assortment.getName();
			Log::log(2, str.c_str());
			assortments.push_back(assortment);
			break;
		}
		default: {
			throw "Unrecognized XML-element: "+tagName.toString();
		}
		}
	}
}

Assortment & AssortmentContainer::get(size_t const index) {
	assert(index < assortments.size());
	return assortments[index];
}

Assortment & AssortmentContainer::getByName(string const & assortmentName) {
	vector<Assortment>::iterator i = assortments.begin(), iend = assortments.end();
	for (; i != iend; ++i) {
		if (i->getName() == assortmentName)
			return *i;
	}
	throw "No such assortment defined: " + assortmentName;
}
