#ifndef COMPARTMENTDATACONTAINER_H_
#define COMPARTMENTDATACONTAINER_H_

#include <vector>
using std::vector;

#ifdef USE_MPI
#include <mpi.h>
#endif

#include <xercesc/util/XercesVersion.hpp>

namespace XERCES_CPP_NAMESPACE {
class DOMElement;
}

class CompartmentData;
class SimulationData;

class CompartmentDataContainer {
public:
	CompartmentDataContainer();
	~CompartmentDataContainer();

#ifdef USE_MPI
	void broadcast(int const rootId, MPI_Comm const comm, SimulationData const & sim);
#endif

	typedef vector<CompartmentData*>::const_iterator const_iterator;
	inline const_iterator begin() const { return compartmentDatas.begin(); }
	inline const_iterator end() const { return compartmentDatas.end(); }

	CompartmentData * find(unsigned int const compartmentId) const;

	inline void add(CompartmentData * const data) {
		compartmentDatas.push_back(data);
	}
	inline CompartmentData * operator[](size_t const index) const {
		return compartmentDatas[index];
	}
	inline size_t size() const {
		return compartmentDatas.size();
	}
	void loadData(XERCES_CPP_NAMESPACE::DOMElement const * compartments, SimulationData const & sim);
	void setupIds();

private:
	vector<CompartmentData*> compartmentDatas;

};

#endif /*COMPARTMENTDATACONTAINER_H_*/
