/*
 * ModelTerm.cpp
 *
 *  Created on: 2008-sep-30
 *      Author: joschori
 */

#include "ModelTerm.h"
#include "StandardNormalRandomNumberGenerator.h"
#include "SecondaryModel.h"
#include "Vector.h"
#include "EvaluatorFunctions.h"
#include "Model.h"

ModelTerm::ModelTerm(SecondaryModelData const coeff) :
	coefficientData(coeff), factors() {

}

ModelTerm::~ModelTerm() {

}

double ModelTerm::calculateCoefficient(Vector & secondaryVariables) const {
	// Avoid unnecessary calculations
	if (coefficientData.hasParams()) {
		return SecondaryModel::calculateMark(secondaryVariables, coefficientData.getTerms());
	} else {
		return 0.0;
	}
}

double ModelTerm::evaluateTerm(Vector & variables, Vector & secondaryVariables) const {
	double value = calculateCoefficient(secondaryVariables);
	if (value == 0.0)
		return 0.0;
	for (vector<ModelFactor>::const_iterator factIter = factors.begin(); factIter != factors.end(); ++factIter) {
		unsigned int const & id = factIter->getVariableId();
		if (id == Model::EPSILON)
			variables[id] = StandardNormalRandomNumberGenerator::getLimited();
		double const base = variables[id];
		double const exponent = factIter->getExponent();
		value *= EvaluatorFunctions::checkingPow(base,exponent);
	}
	return value;
}

ostream & operator<<(ostream & os, ModelTerm const & term) {
	os << term.coefficientData;
	for(vector<ModelFactor>::const_iterator factIter=term.factors.begin(); factIter!=term.factors.end(); ++factIter) {
		os << "*" << *factIter;
	}
	return os;
}
