/*
 * ModelTerm.h
 *
 *  Created on: 2008-sep-30
 *      Author: joschori
 */

#ifndef MODELTERM_H_
#define MODELTERM_H_


#include "ModelFactor.h"
#include "SecondaryModelData.h"

#include <ostream>
using std::ostream;
#include <vector>
using std::vector;

#ifdef USE_MPI
#include "MPIUtil.h"
#endif

class Vector;

class ModelTerm {
public:
	ModelTerm(SecondaryModelData const coeff = SecondaryModelData());
	~ModelTerm();

#ifdef USE_MPI
	inline void broadcast(int const rootId, MPI_Comm const comm) {
		coefficientData.broadcast(rootId, comm);
		unsigned int numFactors = factors.size();
		MPIUtil::broadcast(numFactors, rootId, comm);
		if(MPIUtil::mpiId()!=rootId)
			factors.resize(numFactors);
		for(unsigned int factorIndex=0; factorIndex<numFactors; factorIndex++) {
			factors[factorIndex].broadcast(rootId, comm);
		}
	}
#endif

	inline bool hasParams() const {
		return coefficientData.hasParams();
	}
	inline vector<ModelFactor> const & getFactors() const {
		return factors;
	}
	inline void insert(ModelFactor const & factor) {
		factors.push_back(factor);
	}
	inline void setCoefficient(SecondaryModelData const & data) {
		coefficientData = data;
	}
	inline SecondaryModelData const & getCoefficient() const {
		return coefficientData;
	}
	double calculateCoefficient(Vector & secondaryVariables) const;

	double evaluateTerm(Vector & variables, Vector & secondaryVariables) const;

	friend ostream & operator<<(ostream & os, ModelTerm const & term);

private:
	SecondaryModelData coefficientData;
	vector<ModelFactor> factors;
};

ostream & operator<<(ostream & os, ModelTerm const & term);

#endif /* MODELTERM_H_ */
