#include "MortalityData.h"

#include "XercesString.h"
#include "XercesUtilities.h"
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMElement.hpp>
#include <xercesc/dom/DOMNodeList.hpp>
XERCES_CPP_NAMESPACE_USE

#include <map>
using std::map;

MortalityData::MortalityData()
: regularMortalityThresholdValue(0.0f)
{
}

MortalityData::~MortalityData()
{
}

bool MortalityData::hasRegularTreeMortality() const
{
	return regularMortalityThresholdValue > 0.0f;
}

bool MortalityData::hasIrregularTreeMortality() const
{
	return hasParams();
}

void MortalityData::loadExtraData(DOMElement const * extraData)
{
	// eUnrecognized denotes an unrecognized tag.
	enum Tags { eUnrecognized, eRegularMortalityThreshold };

	map<string, int> tagMap;
	tagMap["regularMortalityThreshold"] = eRegularMortalityThreshold;
	XercesString tagName = XMLString::replicate(extraData->getTagName());
	switch(tagMap[tagName.toString()])
	{
		case(eRegularMortalityThreshold):
		{
			setRegularMortalityThresholdValue(XercesUtilities::getFirstChildTextContentAsDouble(extraData));
			// Log::log(6,"Regular mortality threshold value: %f", getRegularMortalityThresholdValue());
			break;
		}
		default:	// Handles eUnrecognized.
		{
			throw "Unrecognized XML-element: "+tagName.toString();
		}
	}
}
