#ifndef MORTALITYDATA_H_
#define MORTALITYDATA_H_

#include "CommonData.h"
#include "Log.h"

/**
 * Data class for the mortality process.
 */
class MortalityData: public CommonData
{
public:
	MortalityData();
	~MortalityData();
#ifdef USE_MPI
	inline void broadcast(int const rootId, MPI_Comm const comm) {
		CommonData::broadcast(rootId,comm);
		MPIUtil::broadcast(regularMortalityThresholdValue, rootId, comm);
	}
#endif

	inline float getRegularMortalityThresholdValue() const {
		return regularMortalityThresholdValue;
	}
	inline void setRegularMortalityThresholdValue(float value) {
		regularMortalityThresholdValue = value;
	}


	/**
	 * Returns true if this MortalityData specifies that regular tree mortality will occur, otherwise false.
	 */
	bool hasRegularTreeMortality() const;

	/**
	 * Returns true if this MortalityData specifies that irregular tree mortality will occur, otherwise false.
	 */
	bool hasIrregularTreeMortality() const;
	
	void loadExtraData(XERCES_CPP_NAMESPACE::DOMElement const * extraData);
	
private:
	float	regularMortalityThresholdValue;
	
};

#endif /*MORTALITYDATA_H_*/
