#ifndef SECONDARYMODELDATA_H_
#define SECONDARYMODELDATA_H_

#include <ostream>
using std::ostream;

#include "SecondaryModelTerm.h"

#ifdef USE_MPI
#include "MPIUtil.h"
#endif

class SecondaryModelData {
public:
	SecondaryModelData();
	~SecondaryModelData();

#ifdef USE_MPI
	inline void broadcast(int const rootId, MPI_Comm const comm) {
		unsigned int numTerms = terms.size();
		MPIUtil::broadcast(numTerms, rootId, comm);
		if (MPIUtil::mpiId() != rootId)
			terms.resize(numTerms);
		for (unsigned int termIndex = 0; termIndex < numTerms; termIndex++) {
			terms[termIndex].broadcast(rootId, comm);
		}
	}
#endif
	inline vector<SecondaryModelTerm> const & getTerms() const {
		return terms;
	}
	inline void insert(SecondaryModelTerm const & term) {
		terms.push_back(term);
	}
	inline bool hasParams() const {
		for (vector<SecondaryModelTerm>::const_iterator termIter = terms.begin(); termIter != terms.end(); ++termIter) {
			if (termIter->getCoefficient() != 0.0)
				return true;
		}
		return false;
	}
	friend ostream & operator<<(ostream & os, SecondaryModelData const & data);

private:
	vector<SecondaryModelTerm> terms; // beta* and epsilon*

};

ostream & operator<<(ostream & os, SecondaryModelData const & data);

#endif /*SECONDARYMODELDATA_H_*/
