/*
 * SecondaryModelTerm.cpp
 *
 *  Created on: 2008-sep-29
 *      Author: joschori
 */

#include "SecondaryModelTerm.h"
#include "EvaluatorFunctions.h"
#include "Vector.h"
#include "StandardNormalRandomNumberGenerator.h"
#include "SecondaryModel.h"

SecondaryModelTerm::SecondaryModelTerm(double const coef) :
	coefficient(coef) {

}

SecondaryModelTerm::~SecondaryModelTerm() {

}
double SecondaryModelTerm::evaluateTerm(Vector & secondaryVariables) const {
	if (coefficient == 0.0)
		return 0.0;
	double value = coefficient;
	for (vector<SecondaryModelFactor>::const_iterator factIter = factors.begin(); factIter != factors.end(); ++factIter) {
		unsigned int const & id = factIter->getVariableId();
		if (id == SecondaryModel::EPSILON)
			secondaryVariables[id] = StandardNormalRandomNumberGenerator::get();
		double const base = secondaryVariables[id];
		double const exponent = factIter->getExponent();
		value *= EvaluatorFunctions::checkingPow(base, exponent);
	}
	return value;
}
ostream & operator<<(ostream & os, SecondaryModelTerm const & term) {
	os << term.coefficient;
	for (vector<SecondaryModelFactor>::const_iterator factIter = term.factors.begin(); factIter != term.factors.end(); ++factIter) {
		os << "*" << *factIter;
	}
	return os;
}
