/*
 * SecondaryModelTerm.h
 *
 *  Created on: 2008-sep-29
 *      Author: joschori
 */

#ifndef SECONDARYMODELTERM_H_
#define SECONDARYMODELTERM_H_

#include "SecondaryModelFactor.h"

#include <ostream>
using std::ostream;
#include <vector>
using std::vector;

#ifdef USE_MPI
#include "MPIUtil.h"
#endif

class Vector;

class SecondaryModelTerm {
public:
	SecondaryModelTerm(double const coef = 0.0);
	~SecondaryModelTerm();

#ifdef USE_MPI
	inline void broadcast(int const rootId, MPI_Comm const comm) {
		MPIUtil::broadcast(coefficient, rootId, comm);
		unsigned int numFactors = factors.size();
		MPIUtil::broadcast(numFactors, rootId, comm);
		if (MPIUtil::mpiId() != rootId)
			factors.resize(numFactors);
		for (unsigned int factorIndex = 0; factorIndex < numFactors; factorIndex++) {
			factors[factorIndex].broadcast(rootId, comm);
		}
	}
#endif

	inline double const & getCoefficient() const {
		return coefficient;
	}
	inline vector<SecondaryModelFactor> const & getFactors() const {
		return factors;
	}
	inline void insert(SecondaryModelFactor const & factor) {
		factors.push_back(factor);
	}
	double evaluateTerm(Vector & secondaryVariables) const;

	friend ostream & operator<<(ostream & os, SecondaryModelTerm const & term);

private:
	double coefficient;
	vector<SecondaryModelFactor> factors;
};

ostream & operator<<(ostream & os, SecondaryModelTerm const & term);

#endif /* SECONDARYMODELTERM_H_ */
