#include "loadCoefficients.h"

#include "XercesString.h"
#include "XercesUtilities.h"
#include <xercesc/dom/DOMElement.hpp>
#include <xercesc/util/XMLString.hpp>
XERCES_CPP_NAMESPACE_USE

#include <map>
using std::map;
#include <string>
using std::string;

vector<double> loadCoefficients(DOMElement const * coefficients)
{
	vector<double> values;
	
	// eUnrecognized denotes an unrecognized tag.
	enum Tags { eUnrecognized, eCoefficient };
	
	map<string, int> tagMap;
	tagMap["coefficient"] = eCoefficient;
	
	for(DOMElement * element = XercesUtilities::getFirstChildDOMElement(coefficients); element != 0; element = XercesUtilities::getNextSiblingDOMElement(element)) {
		XercesString tagName = XMLString::replicate(element->getTagName());
		switch(tagMap[tagName.toString()])
		{
			case(eCoefficient):
			{
				values.push_back(XercesUtilities::getFirstChildTextContentAsDouble(element));
				break;
			}
			default:	// Handles eUnrecognized.
			{
				throw "Unrecognized XML-element: "+tagName.toString();
			}
		}
	}
	return values;
}
