#ifndef CLEARCUT_H_
#define CLEARCUT_H_

#include "Management.h"
#include "SizeClassThinning.h"

#ifdef USE_MPI
#include "MPIUtil.h"
#endif

#include <vector>
using std::vector;

class Compartment;
class TreeContainer;
class Tree;

class ClearCut: public SizeClassThinning {
public:
	ClearCut();
	virtual ~ClearCut();
	
	virtual MObjectType getObjectType() const { return eCLEARCUTM; }

#ifdef USE_MPI
	virtual void broadcast(int const rootId, MPI_Comm const comm) {
		SizeClassThinning::broadcast(rootId, comm);
		MPIUtil::broadcast(treesPerHectareLeft, rootId, comm);
		MPIUtil::broadcast(forcedDistanceLimit, rootId, comm);
		MPIUtil::broadcast(sparedTreesMinDiameter, rootId, comm);
		MPIUtil::broadcast(sparedTreesMaxDiameter, rootId, comm);
	}
#endif
	virtual void apply(Compartment * compartment, TreeContainer * treeContainer);
	/**
	 * Loads XML data for the Management.
	 * 
	 * Precondition:	@management must not be null.
	 */
	virtual void loadData(XERCES_CPP_NAMESPACE::DOMElement const * management);
	
private:
	unsigned int treesPerHectareLeft;
	double forcedDistanceLimit;
	float sparedTreesMinDiameter;
	float sparedTreesMaxDiameter;

	void findTreesToKeep(vector<Tree*> & validTrees, TreeContainer & treesToKeep, unsigned int const numToKeep, float const dbhLimit) const;
	void markTreesForRemoval(Compartment const * const compartment, TreeContainer & treesToKeep) const;
	void getUnmarkedLivingTrees(Compartment const * const compartment, vector<Tree*> & unMarkedLivingTrees) const;
	void findMinMaxDiameters(vector<Tree*> const & validTrees, float & min, float & max) const;
};

#endif /*CLEARCUT_H_*/
