#include "EnergywoodCutting.h"
#include "Compartment.h"
#include "CompartmentData.h"
#include "TreeContainer.h"
#include "Tree.h"
#include "Log.h"

#include "XercesString.h"
#include "XercesUtilities.h"
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMElement.hpp>
XERCES_CPP_NAMESPACE_USE

#include <map>
using std::map;

EnergywoodCutting::EnergywoodCutting() {
	
}

EnergywoodCutting::~EnergywoodCutting() {
	
}

void EnergywoodCutting::apply(Compartment * compartment, TreeContainer * treeContainer) {
	CompartmentData const * const compData = compartment->getCompartmentData();
	if (!compData->getCompartmentPolygon()->isActive()) {
		return;
	}

	Log::log(4, "Applying energy wood cutting to compartment id %d", compData->getIdNumber());
	for (Tree * curTree = compartment->getFirstTree(); curTree != NULL; curTree = curTree->getNextTreeInCompartment()) {
		if (!curTree->isDead() && !curTree->isMarked() &&
				curTree->getMark(TreeMarks::DIAMETER) <= diameterLimit &&
				shouldCutSpecies(curTree->getSpeciesId())) {
			curTree->markForRemoval();
		}
	}
}

void EnergywoodCutting::loadData(DOMElement const * management) {
	// eUnrecognized denotes an unrecognized tag.
	enum Tags { eUnrecognized, eDiameterLimit };
	
	map<string, int> tagMap;
	tagMap["diameterLimit"] = eDiameterLimit;
	
	for (DOMElement * element = XercesUtilities::getFirstChildDOMElement(management); element != 0; element = XercesUtilities::getNextSiblingDOMElement(element)) {
		XercesString tagName = XMLString::replicate(element->getTagName());
		switch (tagMap[tagName.toString()])
		{
			case(eDiameterLimit):
			{
				diameterLimit = XercesUtilities::getFirstChildTextContentAsDouble(element);
				break;
			}
			default:	// Handles eUnrecognized.
			{
				loadCommonData(element);
			}
		}
	}

}
