#ifndef LOWTHINNING_H_
#define LOWTHINNING_H_

#include "Management.h"
#include "WeibullThinning.h"

#ifdef USE_MPI
#include "MPIUtil.h"
#endif

class Compartment;
class TreeContainer;
class WeibullParameters;

class LowThinning: public WeibullThinning {
public:
	LowThinning();
	virtual ~LowThinning();
	
	virtual MObjectType getObjectType() const { return eLOWTHINM; }

#ifdef USE_MPI
	virtual void broadcast(int const rootId, MPI_Comm const comm) {
		WeibullThinning::broadcast(rootId, comm);
	}
#endif
	/**
	 * Loads XML data for the Management.
	 * 
	 * Precondition:	@management must not be null.
	 */
	virtual void loadData(XERCES_CPP_NAMESPACE::DOMElement const * management);
protected:
	virtual void initIterativeThinning(Compartment * compartment, TreeContainer * treeContainer);

	virtual void applyIteration(
			Compartment * compartment, TreeContainer * treeContainer,
			WeibullFunction const & weibull1, WeibullFunction const & weibull2,
			double const intersectionPoint);
	
private:
	void applyLowThinning(
			Compartment * compartment,
			vector<Tree*> const & classTrees, double const thinningRate);

	vector<float> sizeClassBounds;
};

#endif /*LOWTHINNING_H_*/
