#ifndef MANAGEMENT_H_
#define MANAGEMENT_H_

#include <set>
using std::set;

#include <xercesc/util/XercesVersion.hpp>

namespace XERCES_CPP_NAMESPACE
{
	class DOMElement;
}

class TreeContainer;
class Compartment;

enum MObjectType { eNULLM, eSELECTIVEM, eDIMENSIONM, eENERGYWOODM, eLOWTHINM, eCLEARCUTM };

#ifdef USE_MPI
#include "MPIUtil.h"
#endif

/**
 * Interface for forest management methods.
 */
class Management
{
public:
	Management();
	virtual ~Management();
	virtual MObjectType getObjectType() const = 0;
#ifdef USE_MPI
	inline virtual void broadcast(int const rootId, MPI_Comm const comm) {
		MPIUtil::broadcast(secondaryModelValue, rootId, comm);
		MPIUtil::broadcast(removalPerHectare, rootId, comm);
	}
#endif
	virtual void apply(Compartment * compartment, TreeContainer * treeContainer) = 0;
	static void findMinMaxDiameters(Compartment const * const compartment, double & minDiameter, double & maxDiameter);
	/**
	 * Loads XML data for the Management.
	 * 
	 * Precondition:	@management must not be null.
	 */
	virtual void loadData(XERCES_CPP_NAMESPACE::DOMElement const * management) = 0;

	inline double getSecondaryModelValue() const {
		return secondaryModelValue;
	}
	inline double getRemovalPerHectare() const {
		return removalPerHectare;
	}

protected:
	set<unsigned int> speciesToCut;
	double secondaryModelValue;
	double removalPerHectare;

	void loadCommonData(XERCES_CPP_NAMESPACE::DOMElement const * common);

	inline void setRemovalPerHectare(double const removal) {
		removalPerHectare = removal;
	}

	bool shouldCutSpecies(unsigned int speciesId) const;
};

#endif /*MANAGEMENT_H_*/
