#ifndef MANAGEMENTTEMPLATE_H_
#define MANAGEMENTTEMPLATE_H_

#include <memory>
using std::auto_ptr;
#include <string>
using std::string;
#ifdef USE_MPI
#include <mpi.h>
#endif

#include <xercesc/util/XercesVersion.hpp>

class Management;
class Compartment;
class TreeContainer;

namespace XERCES_CPP_NAMESPACE
{
	class DOMElement;
}

class ManagementTemplate {
public:
	ManagementTemplate(string const & name);
	~ManagementTemplate();

#ifdef USE_MPI
	void broadcast(int const rootId, MPI_Comm const comm);
#endif

	void apply(unsigned int iteration, unsigned int iterationLength, Compartment * compartment, TreeContainer * treeContainer) const;
	void loadData(XERCES_CPP_NAMESPACE::DOMElement const * managementTemplate);
	inline string const & getName() const {
		return templateName;
	}
	double getMethodSecondaryModelValue() const;

private:
	auto_ptr<Management>	method;
	auto_ptr<Management>	clearcut;
	double					lowerBasalArea;
	double					lowerCompartmentVolume;
	unsigned int			thinningIntervalInYears;
	unsigned int			yearsBetweenClearCuts;
	string					templateName;

	void calculateCharacteristicsOfLivingTrees(Compartment const * compartment,
			double & basalArea, double & volume) const;

	void loadManagementMethod(XERCES_CPP_NAMESPACE::DOMElement const * management);

	inline Management * getMethod() const {
		return method.get();
	}
	inline Management * getClearCut() const {
		return clearcut.get();
	}
	inline unsigned int getYearsBetweenClearCuts() const {
		return yearsBetweenClearCuts;
	}
	inline double getLowerBasalArea() const {
		return lowerBasalArea;
	}
	inline double getLowerCompartmentVolume() const {
		return lowerCompartmentVolume;
	}
	inline unsigned int getThinningIntervalInYears() const {
		return thinningIntervalInYears;
	}

};

#endif /*MANAGEMENTTEMPLATE_H_*/
