#ifndef SELECTIVETHINNING_H_
#define SELECTIVETHINNING_H_

#include <vector>
using std::vector;

#ifdef USE_MPI
#include "MPIUtil.h"
#endif

#include "WeibullThinning.h"

class TreeContainer;

class SelectiveThinning: public WeibullThinning {
public:
	SelectiveThinning();
	virtual ~SelectiveThinning();
	inline virtual MObjectType getObjectType() const { return eSELECTIVEM; }
#ifdef USE_MPI
	inline virtual void broadcast(int const rootId, MPI_Comm const comm) {
		WeibullThinning::broadcast(rootId, comm);
		MPIUtil::broadcast(thinningLevel, rootId, comm);
		MPIUtil::broadcast(densityCoefficient, rootId, comm);
	}
#endif
	/**
	 * Loads XML data for the SelectiveThinning.
	 *
	 * Precondition:	@management must not be null.
	 */
	virtual void loadData(XERCES_CPP_NAMESPACE::DOMElement const * management);
protected:
	virtual void initIterativeThinning(Compartment * compartment, TreeContainer * treeContainer);
	
	virtual void applyIteration(
			Compartment * compartment, TreeContainer * treeContainer,
			WeibullFunction const & weibull1, WeibullFunction const & weibull2,
			double const intersectionPoint);
private:
	void applyStochasticThinningWithFixedProbabilities(
			Compartment * compartment, TreeContainer * trees,
			vector<Tree*> const & classTrees,
			double const thinningRate);

	double thinningLevel;
	double densityCoefficient;
	vector<float> sizeClassBounds;
};

#endif /*SELECTIVETHINNING_H_*/

