#ifdef USE_MPI

#ifndef COMPARTMENTCOMMUNICATOR_H_
#define COMPARTMENTCOMMUNICATOR_H_

#include <stdint.h>
#include <vector>
using std::vector;

class Vector;
#include "CompartmentData.h"
#include "MPIUtil.h"
#include "Log.h"

#ifdef COMPARISONSEEDING
class TreeContainer;
class Compartment;
class Tree;
#include <mpi.h>
#endif

class CompartmentCommunicator {
public:
	static void combineAverage(CompartmentData const * const compartmentData, unsigned int & averageContributions,
			Vector & averages);

	template<class T> static void sum(unsigned int compartmentId, T & values) {
		CompartmentData const * const thisCompartmentData = findCompartmentData(compartmentId);
		if (thisCompartmentData != NULL) {
			const MPI_Comm compartmentCommunicator = thisCompartmentData->getCompartmentCommunicator();
			MPIUtil::sum(values, compartmentCommunicator);
		} else {
			Log::logError(-1,"CompartmentCommunicator::sum(compartmentId,templateclass) could not resolve compartmentData for compartmentId %u", compartmentId);
			throw "CompartmentCommunicator::sum(compartmentId,templateclass) could not resolve compartmentData";
		}
	}

	static void max(unsigned int compartmentId, unsigned int & value);
	static CompartmentData const * findCompartmentData(unsigned int compartmentId);
#ifdef COMPARISONSEEDING
	static void transferCompartmentTrees(TreeContainer & allTrees, Compartment * c, vector<Tree*> & receivedTrees);
	static void transferTrees(MPI_Comm comm, vector<Tree*> const & sendTrees, vector<Tree*> & receivedTrees);
#endif
};

#endif /*COMPARTMENTCOMMUNICATOR_H_*/

#endif /* USE_MPI */
