#ifdef USE_MPI

#ifndef MPIUTIL_H_
#define MPIUTIL_H_

#include <mpi.h>

#include <stdint.h>
#include <vector>
using std::vector;
#include <string>
using std::string;

class ForestSplitter;
class Vector;

class MPIUtil {
private:
	static MPI_Datatype MPI_Treestruct;

	static void registerMPIType();

	static inline void freeMPIType() {
		if (MPI_Treestruct != MPI_DATATYPE_NULL)
			MPI_Type_free(&MPI_Treestruct);
	}

	static int staticMpiId, staticMpiSize;
	static size_t numberOfBroadcastsDone;
	static size_t numberOfSumsDone;

public:
	static void cleanup() {
		freeMPIType();
	}
	static inline MPI_Datatype getMPITreeStruct() {
		if (MPI_Treestruct == MPI_DATATYPE_NULL) {
			MPIUtil::registerMPIType();
		}
		return MPI_Treestruct;
	}
	static void sum(Vector & v, MPI_Comm const communicator = MPI_COMM_WORLD);
	static void sum(vector<double> & v, MPI_Comm const communicator = MPI_COMM_WORLD);
	static void sum(vector<int> & v, MPI_Comm const communicator = MPI_COMM_WORLD);
	static void sum(vector<uint64_t> & v, MPI_Comm const communicator = MPI_COMM_WORLD);
	static void sum(vector<unsigned int> & v, MPI_Comm const communicator = MPI_COMM_WORLD);
	static void sum(int & integer, MPI_Comm const communicator = MPI_COMM_WORLD);
	static void sum(uint64_t & integer, MPI_Comm const communicator = MPI_COMM_WORLD);
	static void sum(unsigned int & integer, MPI_Comm const communicator = MPI_COMM_WORLD);
	static void sum(double & value, MPI_Comm const communicator = MPI_COMM_WORLD);
	static void min(double & lowerBound, MPI_Comm const communicator = MPI_COMM_WORLD);
	static void max(double & upperBound, MPI_Comm const communicator = MPI_COMM_WORLD);
	static void max(unsigned int & upperBound, MPI_Comm const communicator = MPI_COMM_WORLD);
	static void allgather(uint64_t const & integer, vector<uint64_t> & integers, MPI_Comm const communicator = MPI_COMM_WORLD);
	static void broadcast(unsigned char & value, int const rootId, MPI_Comm const communicator = MPI_COMM_WORLD);
	static void broadcast(unsigned int & value, int const rootId, MPI_Comm const communicator = MPI_COMM_WORLD);
	static void broadcast(int & value, int const rootId, MPI_Comm const communicator = MPI_COMM_WORLD);
	static void broadcast(float & value, int const rootId, MPI_Comm const communicator = MPI_COMM_WORLD);
	static void broadcast(double & value, int const rootId, MPI_Comm const communicator = MPI_COMM_WORLD);
	static void broadcast(bool & value, int const rootId, MPI_Comm const communicator = MPI_COMM_WORLD);
	static void broadcast(vector<string> & strings, int const rootId, MPI_Comm const communicator = MPI_COMM_WORLD);
	static void broadcast(string & str, int const rootId, MPI_Comm const communicator = MPI_COMM_WORLD);
	static void broadcast(Vector & values, int const rootId, MPI_Comm const communicator = MPI_COMM_WORLD);
	static void broadcast(vector<double> & values, int const rootId, MPI_Comm const communicator = MPI_COMM_WORLD);
	static void broadcast(vector<float> & values, int const rootId, MPI_Comm const communicator = MPI_COMM_WORLD);
	static void broadcast(vector<int> & values, int const rootId, MPI_Comm const communicator = MPI_COMM_WORLD);
	static void broadcast(vector<unsigned int> & values, int const rootId, MPI_Comm const communicator = MPI_COMM_WORLD);

	static inline MPI_Comm getActiveCommunicator() {
		return MPI_COMM_WORLD;
	}
	static inline int mpiId() {
		return MPIUtil::staticMpiId;
	}
	static inline int mpiSize() {
		return MPIUtil::staticMpiSize;
	}
	static inline void findMpiId() {
		MPI_Comm_rank(MPI_COMM_WORLD, &MPIUtil::staticMpiId);
	}
	static inline void findMpiSize(MPI_Comm const communicator) {
		MPI_Comm_size(communicator, &MPIUtil::staticMpiSize);
	}
	static inline size_t getNumberOfSumsDone() {
		return MPIUtil::numberOfSumsDone;
	}
	static inline size_t getNumberOfBroadcastsDone() {
		return MPIUtil::numberOfBroadcastsDone;
	}

	static const int moreToTransfer = 1; // randomly chosen tag number :)
	static const int lastTransfer = 2; // randomly chosen tag number :)

	friend class ForestSplitter;
};

#endif /*MPIUTIL_H_*/

#endif /*USE_MPI*/
