#ifdef USE_MPI

#ifndef TREESENDER_H_
#define TREESENDER_H_

#include "TreeMarks.h"
#include "Tree.h"

// If the Treestruct is changed, remember to update MPIUtil::registerMPIType() too!
struct Treestruct {
	double x, y;
	float marks[TreeMarks::NUMBER_OF_MARKS], previousDbhGrowth[2], initialHeterogeneity;
#ifdef INCLUDE_ENVIRONMENTAL_EFFECTS
	float randomNumbers[Tree::NUMBER_OF_RND_NUM_SETS * TreeMarks::NUMBER_OF_MARKS];
	float processX, processY;
#endif
	unsigned int speciesId, age, compartmentId;
};

class TreeContainer;
class Tree;

#include "ForestSplitter.h"

class TreeSender {
public:
	static vector<Tree*> transferForeignTrees(TreeContainer & allTrees);
	static void rebalanceTreesFast(TreeContainer & allTrees);

	static Treestruct createBufferFromTree(Tree const * tree);
	static Tree * createTreeFromBuffer(Treestruct const & in);

	static size_t BUFFSIZE;

private:
	static void transferTreesInDirection(map<Direction, vector<Tree const *> > const & treesToSend, vector<Tree*> & acceptedReceivedTrees, Direction dir);
	inline static bool treeIsRelevant(Treestruct const & tstruct, Point const & minCoord, Point const & maxCoord, float limit) {
		return tstruct.x <= (maxCoord.x+limit) && tstruct.x >= (minCoord.x-limit) && tstruct.y <= (maxCoord.y+limit) && tstruct.y >= (minCoord.y-limit);
	}
	static void setupTreesToSend(TreeContainer const & allTrees, map<Direction, vector<Tree const *> > & treesToSend);

};

#endif /*TREESENDER_H_*/
#endif // USE_MPI
