#ifndef CLUSTEREDPOINTGENERATOR_H_
#define CLUSTEREDPOINTGENERATOR_H_

#include "PointGenerator.h"

class ClusteredPointGenerator: public PointGenerator
{
public:
	ClusteredPointGenerator();
	virtual ~ClusteredPointGenerator();
	virtual PGObjectType getObjectType() const = 0;
#ifdef USE_MPI
	virtual void broadcast(int const rootId, MPI_Comm const comm) = 0;
#endif

	virtual void generatePointsInside(Polygon const & polygon, double const percent, vector<Point> & offspringPoints) const;
	/**
	 * Loads XML data for the ClusteredPointGenerator.
	 * 
	 * Precondition:	@pointGenerator must not be null.
	 */
	virtual void loadData(XERCES_CPP_NAMESPACE::DOMElement const * pointGenerator);
	
	inline int getExpectedNumberOfOffspringPoints() const {
		return expectedNumberOfOffspringPoints;
	}
	inline void setExpectedNumberOfOffspringPoints(int numberOfPoints) {
		expectedNumberOfOffspringPoints = numberOfPoints;
	}
	inline double getOffspringDispersionRadius() const {
		return offspringDispersionRadius;
	}
	inline void setOffspringDispersionRadius(double radius) {
		offspringDispersionRadius = radius;
	}
private:
	virtual PointGenerator const & getPointGenerator() const = 0;
	/**
	 * Provides a way for passing unrecognized XML elements to derivatives of ClusteredPointGenerator.
	 * 
	 * Precondition:	@element must not be null.
	 */
	virtual void loadDatum(XERCES_CPP_NAMESPACE::DOMElement const * element) = 0;
	
	void generateOffspringPoints(Polygon const & polygon, vector<Point> & motherPoints, vector<Point> & offspringPoints) const;
	
protected:
	int		expectedNumberOfOffspringPoints;
	double	offspringDispersionRadius;
};

#endif /*CLUSTEREDPOINTGENERATOR_H_*/
