#include "HomogenousClusteredPointGenerator.h"
#include "Log.h"

#include "XercesString.h"
#include "XercesUtilities.h"
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMElement.hpp>
XERCES_CPP_NAMESPACE_USE

#include <map>
using std::map;

HomogenousClusteredPointGenerator::HomogenousClusteredPointGenerator()
{
}

HomogenousClusteredPointGenerator::~HomogenousClusteredPointGenerator()
{
}

PointGenerator const & HomogenousClusteredPointGenerator::getPointGenerator() const
{
	return pointGenerator;
}

void HomogenousClusteredPointGenerator::loadDatum(DOMElement const * element)
{
	// eUnrecognized denotes an unrecognized tag.
	enum Tags { eUnrecognized, eExpectedNumberOfMotherPointsPerHectare };
	
	map<string, int> tagMap;
	tagMap["expectedNumberOfMotherPointsPerHectare"] = eExpectedNumberOfMotherPointsPerHectare;
	
	XercesString tagName = XMLString::replicate(element->getTagName());
	switch(tagMap[tagName.toString()])
	{
		case(eExpectedNumberOfMotherPointsPerHectare):
		{
			pointGenerator.setExpectedNumberOfPointsPerHectare(XercesUtilities::getFirstChildTextContentAsInt(element));
//			Log::log(5,"Expected number of mother points per hectare: %d", pointGenerator.getExpectedNumberOfPointsPerHectare());
			break;
		}
		default:	// Handles eUnrecognized.
		{
			throw "Unrecognized XML-element: "+tagName.toString();
		}
	}
}
