#ifndef HOMOGENOUSPOISSONPOINTGENERATOR_H_
#define HOMOGENOUSPOISSONPOINTGENERATOR_H_

namespace XERCES_CPP_NAMESPACE
{
	class DOMElement;
}

#include "PointGenerator.h"

#ifdef USE_MPI
#include "MPIUtil.h"
#endif

class HomogenousPoissonPointGenerator: public PointGenerator
{
public:
    HomogenousPoissonPointGenerator();
    virtual ~HomogenousPoissonPointGenerator();
	inline virtual PGObjectType getObjectType() const {
		return eHOMOPOISPG;
	}
#ifdef USE_MPI
	inline virtual void broadcast(int const rootId, MPI_Comm const comm) {
		MPIUtil::broadcast(expectedNumberOfPointsPerHectare, rootId, comm);
	}
#endif

	virtual void generatePointsInside(Polygon const & shape, double const percent, vector<Point> & points) const;
	/**
	 * Loads XML data for the HomogenousPoissonPointGenerator.
	 * 
	 * Precondition:	@pointGenerator must not be null.
	 */
	virtual void loadData(XERCES_CPP_NAMESPACE::DOMElement const * pointGenerator);
	
	/**
	 * Returns a point which lies inside the specified shape.
	 */
    Point generatePointInside(Polygon const & shape) const;
	inline unsigned int getExpectedNumberOfPointsPerHectare() const {
		return expectedNumberOfPointsPerHectare;
	}
	inline void setExpectedNumberOfPointsPerHectare(unsigned int numberOfPoints) {
		expectedNumberOfPointsPerHectare = numberOfPoints;
	}
private:
	unsigned int	expectedNumberOfPointsPerHectare;
};

#endif /*HOMOGENOUSPOISSONPOINTGENERATOR_H_*/
