#include "InhomogenousClusteredPointGenerator.h"
#include "Log.h"

#include "loadCoefficients.h"
#include "XercesString.h"
#include "XercesUtilities.h"
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMElement.hpp>
XERCES_CPP_NAMESPACE_USE

#include <map>
using std::map;

InhomogenousClusteredPointGenerator::InhomogenousClusteredPointGenerator()
{
}

InhomogenousClusteredPointGenerator::~InhomogenousClusteredPointGenerator()
{
}

PointGenerator const & InhomogenousClusteredPointGenerator::getPointGenerator() const
{
	return pointGenerator;
}

void InhomogenousClusteredPointGenerator::loadDatum(DOMElement const * element)
{
	// eUnrecognized denotes an unrecognized tag.
	enum Tags { eUnrecognized, eExpectedNumberOfMotherPointsPerHectare, eEstimatedPoissonRegressionCoefficientsForMotherPoints };
	
	map<string, int> tagMap;
	tagMap["expectedNumberOfMotherPointsPerHectare"] = eExpectedNumberOfMotherPointsPerHectare;
	tagMap["estimatedPoissonRegressionCoefficientsForMotherPoints"] = eEstimatedPoissonRegressionCoefficientsForMotherPoints;
	
	XercesString tagName = XMLString::replicate(element->getTagName());
	switch(tagMap[tagName.toString()])
	{
		case(eExpectedNumberOfMotherPointsPerHectare):
		{
			pointGenerator.setExpectedNumberOfPointsPerHectare(XercesUtilities::getFirstChildTextContentAsInt(element));
//			Log::log(5,"Expected number of mother points per hectare: %d", pointGenerator.getExpectedNumberOfPointsPerHectare());
			break;
		}
		case(eEstimatedPoissonRegressionCoefficientsForMotherPoints):
		{
			pointGenerator.setEstimatedPoissonRegressionCoefficients(loadCoefficients(element));
//			Log::log(5,"Estimated Poisson regression coefficients loaded");
//			cout << "Estimated Poisson regression coefficients:\n" << pointGenerator.getEstimatedPoissonRegressionCoefficients() << "\n";
			break;
		}
		default:	// Handles eUnrecognized.
		{
			throw "Unrecognized XML-element: "+tagName.toString();
		}
	}
}
