#ifndef INHOMOGENOUSPOISSONPOINTGENERATOR_H_
#define INHOMOGENOUSPOISSONPOINTGENERATOR_H_

#include "PointGenerator.h"
#include "Vector.h"

#ifdef USE_MPI
#include "MPIUtil.h"
#endif

class InhomogenousPoissonPointGenerator: public PointGenerator
{
public:
	enum NumberOfCoefficients { POISSON_REGRESSION=6 };
	
	InhomogenousPoissonPointGenerator();
	virtual ~InhomogenousPoissonPointGenerator();
	inline virtual PGObjectType getObjectType() const {
		return eINHOPOISPG;
	}
#ifdef USE_MPI
	inline virtual void broadcast(int const rootId, MPI_Comm const comm) {
		MPIUtil::broadcast(expectedNumberOfPointsPerHectare, rootId, comm);
		MPIUtil::broadcast(poissonRegressionCoefficients, rootId, comm);
	}
#endif

	virtual void generatePointsInside(Polygon const & shape, double const percent, vector<Point> & points) const;
	/**
	 * Loads XML data for the InhomogenousPoissonPointGenerator.
	 * 
	 * Precondition:	@pointGenerator must not be null.
	 */
	virtual void loadData(XERCES_CPP_NAMESPACE::DOMElement const * pointGenerator);
	
	inline unsigned int getExpectedNumberOfPointsPerHectare() const {
		return expectedNumberOfPointsPerHectare;
	}
	inline void setExpectedNumberOfPointsPerHectare(unsigned int numberOfPoints) {
		expectedNumberOfPointsPerHectare = numberOfPoints;
	}
	inline Vector const & getEstimatedPoissonRegressionCoefficients() const {
		return poissonRegressionCoefficients;
	}
	inline void setEstimatedPoissonRegressionCoefficients(Vector const & coefficients) {
		if(coefficients.size() == POISSON_REGRESSION) {
			poissonRegressionCoefficients = coefficients;
		} else {
			throw "Bad number of estimated Poisson regression coefficients.";
		}
	}
private:
	double supLambda(double, double) const;
	
	unsigned int	expectedNumberOfPointsPerHectare;
	Vector	poissonRegressionCoefficients;
};

#endif /*INHOMOGENOUSPOISSONPOINTGENERATOR_H_*/
