#include "InhomogenousRegularPointGenerator.h"
#include "Point.h"
#include "Log.h"

#include "loadCoefficients.h"
#include "XercesString.h"
#include "XercesUtilities.h"
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMElement.hpp>
XERCES_CPP_NAMESPACE_USE

#include <map>
using std::map;

InhomogenousRegularPointGenerator::InhomogenousRegularPointGenerator()
: elevationSurfaceModelCoefficients(ELEVATION_SURFACE_MODEL)
{
}

InhomogenousRegularPointGenerator::~InhomogenousRegularPointGenerator()
{
}

double InhomogenousRegularPointGenerator::calculatePointPotential(vector<Point> const & points) const
{
	double potential = 1.0;
	double new_x_h = evaluate(points[0]);
	for(unsigned int k = 1; k < points.size(); ++k) {
		double old_x_h = evaluate(points[k]);
		double distance = points[0].distanceTo(points[k]) + old_x_h + new_x_h;
		if(distance < getInteractionRadius()) {
			potential *= getDensity();
		}
	}
	return potential;
}

double InhomogenousRegularPointGenerator::evaluate(Point const & point) const
{
	Vector x(6, 0.0);
	x[0]=1.0;
	x[1]=point.x;
	x[2]=point.y;
	x[3]=point.x*point.y;
	x[4]=point.x*point.x;
	x[5]=point.y*point.y;
	
	return Vector::dotProduct(x, elevationSurfaceModelCoefficients);
}

void InhomogenousRegularPointGenerator::loadDatum(DOMElement const * element)
{
	// eUnrecognized denotes an unrecognized tag.
	enum Tags { eUnrecognized, eEstimatedElevationSurfaceModelCoefficients };
	
	map<string, int> tagMap;
	tagMap["estimatedElevationSurfaceModelCoefficients"] = eEstimatedElevationSurfaceModelCoefficients;
	
	XercesString tagName = XMLString::replicate(element->getTagName());
	switch(tagMap[tagName.toString()])
	{
		case(eEstimatedElevationSurfaceModelCoefficients):
		{
			setElevationSurfaceModelCoefficients(loadCoefficients(element));
//			Log::log(5,"Estimated elevation surface model coefficients loaded");
//			cout << "Estimated elevation surface model coefficients:\n" << getElevationSurfaceModelCoefficients() << "\n";
			break;
		}
		default:	// Handles eUnrecognized.
		{
			throw "Unrecognized XML-element: "+tagName.toString();
		}
	}
}
