#ifndef INHOMOGENOUSREGULARPOINTGENERATOR_H_
#define INHOMOGENOUSREGULARPOINTGENERATOR_H_

#include "RegularPointGenerator.h"
#include "Vector.h"

#ifdef USE_MPI
#include "MPIUtil.h"
#endif

class InhomogenousRegularPointGenerator: public RegularPointGenerator
{
public:
	enum NumberOfCoefficients { ELEVATION_SURFACE_MODEL = 6 };
		
	InhomogenousRegularPointGenerator();
	virtual ~InhomogenousRegularPointGenerator();
	inline virtual PGObjectType getObjectType() const {
		return eINHOREGUPG;
	}
#ifdef USE_MPI
	inline virtual void broadcast(int const rootId, MPI_Comm const comm) {
		MPIUtil::broadcast(elevationSurfaceModelCoefficients, rootId, comm);
		MPIUtil::broadcast(density, rootId, comm);
		MPIUtil::broadcast(interactionRadius, rootId, comm);
		homogenousPoissonPointGenerator.broadcast(rootId,comm);
	}
#endif

	inline Vector const & getElevationSurfaceModelCoefficients() const {
		return elevationSurfaceModelCoefficients;
	}
	inline void setElevationSurfaceModelCoefficients(Vector const & coefficients) {
		if(coefficients.size() == ELEVATION_SURFACE_MODEL) {
			elevationSurfaceModelCoefficients = coefficients;
		} else {
			throw "Bad number of elevation surface model coefficients.";
		}
	}
private:
	virtual double calculatePointPotential(vector<Point> const & points) const;
	/**
	 * Loads one XML element.
	 * 
	 * Precondition:	@element must not be null.
	 */
	virtual void loadDatum(XERCES_CPP_NAMESPACE::DOMElement const * element);
	
	double evaluate(Point const & point) const;
	
	Vector	elevationSurfaceModelCoefficients;
};

#endif /*INHOMOGENOUSREGULARPOINTGENERATOR_H_*/
