#ifndef POINT_H_
#define POINT_H_

#include <cmath>
#include <iostream>

class Point {
public:
    Point() {}
    Point(double xx, double yy): x(xx), y(yy) {}
    /**
     * Returns the distance to another point.
     */
    inline double distanceTo(Point const & point) const {
    	return sqrt(squaredDistanceTo(point));
    }
    inline double squaredDistanceTo(Point const & point) const {
    	double const diff1 = x - point.x;
    	double const squared1 = diff1*diff1;
    	double const diff2 = y - point.y;
    	double const squared2 = diff2*diff2;
    	return squared1+squared2;
    }
    
    double x,y;

    inline bool operator<=(const Point & other) const {
        return (x <= other.x && y <= other.y);
    }

    inline bool operator>=(const Point & other) const {
        return (x >= other.x && y >= other.y);
    }
    
    inline bool operator==(const Point & other) const {
    	return ( (x == other.x) && (y == other.y) );
    }

    inline bool operator!=(const Point & other) const {
    	return ( (x != other.x) || (y != other.y) );
    }

    inline bool operator<(const Point & other) const {
        return (x < other.x && y < other.y);
    }

    inline bool operator>(const Point & other) const {
        return (x > other.x && y > other.y);
    }

    inline Point operator-(const Point & other) const {
    	return Point(x-other.x,y-other.y);
    }

    friend std::ostream& operator<<(std::ostream& s, const Point & p );

};

std::ostream& operator<<(std::ostream& s, const Point & p);

#endif /*POINT_H_*/
