#ifndef POINTGENERATOR_H_
#define POINTGENERATOR_H_

#include <vector>
using std::vector;

#include <xercesc/util/XercesVersion.hpp>

namespace XERCES_CPP_NAMESPACE
{
	class DOMElement;
}

class Point;
class Polygon;

enum PGObjectType { eNULLPG, eHOMOCLUSPG, eHOMOPOISPG, eHOMOREGUPG, eINHOCLUSPG, eINHOPOISPG, eINHOREGUPG };

#ifdef USE_MPI
#include <mpi.h>
#endif

class PointGenerator
{
public:
	PointGenerator();
	virtual ~PointGenerator();
	virtual PGObjectType getObjectType() const = 0;
#ifdef USE_MPI
	virtual void broadcast(int const rootId, MPI_Comm const comm) = 0;
#endif
	/**
	 * Generates points inside @shape and stores them in @points.
	 */
	virtual void generatePointsInside(Polygon const & polygon, double const percent, vector<Point> & points) const = 0;
	/**
	 * Loads XML data for the PointGenerator.
	 * 
	 * Precondition:	@pointGenerator must not be null.
	 */
	virtual void loadData(XERCES_CPP_NAMESPACE::DOMElement const * pointGenerator) = 0;
};

#endif /*POINTGENERATOR_H_*/
