#ifndef REGULARPOINTGENERATOR_H_
#define REGULARPOINTGENERATOR_H_

#include "PointGenerator.h"
#include "HomogenousPoissonPointGenerator.h"

class RegularPointGenerator: public PointGenerator
{
public:
	RegularPointGenerator();
	virtual ~RegularPointGenerator();
	virtual PGObjectType getObjectType() const = 0;
#ifdef USE_MPI
	virtual void broadcast(int const rootId, MPI_Comm const comm) = 0;
#endif

	virtual void generatePointsInside(Polygon const & polygon, double const percent, vector<Point> & points) const;
	/**
	 * Loads XML data for the RegularPointGenerator.
	 * 
	 * Precondition:	@pointGenerator must not be null.
	 */
	virtual void loadData(XERCES_CPP_NAMESPACE::DOMElement const * pointGenerator);
	
	inline double getDensity() const {
		return density;
	}
	inline void setDensity(double den) {
		density = den;
	}
	inline double getInteractionRadius() const {
		return interactionRadius;
	}
	inline void setInteractionRadius(double radius) {
		interactionRadius = radius;
	}
private:
	virtual double calculatePointPotential(vector<Point> const & points) const = 0;
	/**
	 * Provides a way for passing unrecognized XML elements to derivatives of RegularPointGenerator.
	 * 
	 * Precondition:	@element must not be null.
	 */
	virtual void loadDatum(XERCES_CPP_NAMESPACE::DOMElement const * element) = 0;
	
protected:
	double							density;
	double							interactionRadius;
	HomogenousPoissonPointGenerator	homogenousPoissonPointGenerator;
};

#endif /*REGULARPOINTGENERATOR_H_*/
