#ifndef POISSONRANDOMNUMBERGENERATOR_H_
#define POISSONRANDOMNUMBERGENERATOR_H_

#define CTAB 1000		//Number of entries to the cumulative table.
struct Cumul {
	Cumul * p0;			//Pointer to next lower cumulative probability.
	double c;			//Probability of $n$ or fewer events occurring.
	unsigned int n;		//Number of events $n$.
};

class PoissonRandomNumberGenerator {
public:
	PoissonRandomNumberGenerator();
	virtual ~PoissonRandomNumberGenerator();
	static unsigned int get(double const mean);
	static void Destroy();

private:
	// Variables for Poisson random number generator with tables
	static double plambda;				//Previous mean.
	static double epsilon;
	static struct Cumul * cumul;		//Cumulative probability table.
	static struct Cumul * ctab[CTAB];	//Pointers to cumulative probabilities.
	static bool init;
	static unsigned int tableBased(double const lambda);
	static void initial(double const lambda);

	// Variables for Poisson random number generator with rejection formula
	static double oldMean;
	static double almean;
	static double g;
	static double sq;
	static unsigned int rejectionBased(double const mean);
	static double gamma(double const xx);

};

#endif /*POISSONRANDOMNUMBERGENERATOR_H_*/
