#include "StandardNormalRandomNumberGenerator.h"
#include "RandomNumber.h"
#include "Log.h"

using std::sqrt;
using std::log;

double StandardNormalRandomNumberGenerator::limitingFactor = 4.0;
double StandardNormalRandomNumberGenerator::v2 = 0.0;
// Should be positive to prevent numeric overflow.
double StandardNormalRandomNumberGenerator::epsilon = 0.000001;
int StandardNormalRandomNumberGenerator::init = 0;

StandardNormalRandomNumberGenerator::StandardNormalRandomNumberGenerator() {

}

StandardNormalRandomNumberGenerator::~StandardNormalRandomNumberGenerator() {
}

double StandardNormalRandomNumberGenerator::get() {
	/**
	 * This implementation based on implementation by
	 * Clarence Lehman, December 1996
	 * http://www.cedarcreek.umn.edu/tools/utility/rnormal.d.pdf
	 */

#ifdef COMPARISONSEEDING
	init = 0;
#endif
	
	double v1, s;
	if (init == 0) {
		init = 1;
		v2 = 2.0 * RandomNumber::get() - 1.0;
	}
	// Generate a point within the unit circle.
	while (true) {
		v1 = v2;
		v2 = 2.0 * RandomNumber::get() - 1.0;
		s = v1 * v1 + v2 * v2;
		if (epsilon < s && s < 1.0)
			break;
	}
	// Return the normal deviate.
	return (v1 * sqrt(-2.0 * log(s) / s));
}

double StandardNormalRandomNumberGenerator::getLimited() {
	double randomNumber = get();
	while (std::abs(randomNumber) > limitingFactor) {
		randomNumber = get();
	}
	return randomNumber;
}

#ifdef USE_MPI
void StandardNormalRandomNumberGenerator::broadcast(int const rootId, MPI_Comm const comm) {
	MPIUtil::broadcast(limitingFactor, rootId, comm);
}
#endif
