#include "UniformRandomNumberGenerator.h"
#include "Log.h"
#include "Tree.h"
#include "roundToInt.h"

#define SIMPLE_SPRNG

#if defined(COMPARISONSEEDING) && defined(USE_MPI)
	#undef USE_MPI
	#include "sprng_cpp.h"
	#define USE_MPI
#else
	#include "sprng_cpp.h"
#endif

#include <cstdlib>
using std::abs;
#include <limits>
using std::numeric_limits;
#include <cmath>
using std::fmod;

double UniformRandomNumberGenerator::newSeed = 42.0;

UniformRandomNumberGenerator::UniformRandomNumberGenerator() {

}

UniformRandomNumberGenerator::~UniformRandomNumberGenerator() {

}

double UniformRandomNumberGenerator::get() {
	return sprng();
}

void UniformRandomNumberGenerator::setSeed(double const seed) {
	// Convert the seed to an integer and confine it to the interval 0 -- 2^31-1
	int const fixedSeed = abs(roundToInt(fmod(seed, numeric_limits<int>::max())));
	if (init_sprng(fixedSeed, SPRNG_DEFAULT, SPRNG_LCG64) == NULL) {
		Log::logError(-1, "Initialization of SPRNG failed with seed %d, param %d and generator id %d", fixedSeed,
				SPRNG_DEFAULT, SPRNG_LCG64);
	}
}


#ifdef COMPARISONSEEDING
void UniformRandomNumberGenerator::setSeeds(Tree const * const t, double const value) {
	setSeed(t->getMark(TreeMarks::DIAMETER) * value + t->getX()*58234.1 + t->getY());
}
#endif

#ifdef USE_MPI
void UniformRandomNumberGenerator::broadcast(int const rootId, MPI_Comm const comm) {
	MPIUtil::broadcast(newSeed, rootId, comm);
}
#endif
