#ifndef MARKDISTRIBUTION_H_
#define MARKDISTRIBUTION_H_

#include <vector>
using std::vector;

#include <xercesc/util/XercesVersion.hpp>

namespace XERCES_CPP_NAMESPACE {
class DOMElement;
}

/**
 * Classes implementing this interface are used to build a cumulative
 * distribution according to the algorithm and methodology the deriving class
 * defines.
 */
class MarkDistribution {
public:
	virtual ~MarkDistribution() {
	}

	/**
	 * Load parameters from XML.
	 * @param distribution Should be a child element of the <transformation> element
	 */
	virtual void loadData(XERCES_CPP_NAMESPACE::DOMElement const * distribution) = 0;

	/**
	 * Build the distribution and put the result in the variables used as
	 * function arguments.
	 * @param classes x values (classes) of the CDF
	 * @param values  F(x) values of the CDF
	 */
	virtual void setUp(vector<double> & classes, vector<double> & values) = 0;
};

#endif

