#include "MarkTransformation.h"

#include "Log.h"
#ifdef USE_MPI
#include "MPIUtil.h"
#endif

#include "XercesString.h"
#include "XercesUtilities.h"
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMElement.hpp>
XERCES_CPP_NAMESPACE_USE

#include <map>
using std::map;
#include <memory>
using std::auto_ptr;

#include "MarkNormalDistribution.h"
#include "MarkWeibullDistribution.h"
#include "MarkECDFDistribution.h"
#include "MarkDataDistribution.h"

MarkTransformation::MarkTransformation() :
	loaded(false) {

}

MarkTransformation::~MarkTransformation() {
}

#ifdef USE_MPI
void MarkTransformation::broadcast(int const rootId, MPI_Comm const comm) {
	MPIUtil::broadcast(loaded, rootId, comm);
	MPIUtil::broadcast(targetCDFClasses, rootId, comm);
	MPIUtil::broadcast(targetCDFValues, rootId, comm);
}
#endif

void MarkTransformation::loadData(XERCES_CPP_NAMESPACE::DOMElement const * transformation) {
	// eUnrecognized denotes an unrecognized tag.
	enum Tags {eUnrecognized, eNormalDistribution, eWeibullDistribution, eEmpiricalCumulativeDistribution, eEmpiricalData};

	map<string, int> tagMap;
	tagMap["normalDistribution"] = eNormalDistribution;
	tagMap["weibullDistribution"] = eWeibullDistribution;
	tagMap["empiricalCumulativeDistribution"] = eEmpiricalCumulativeDistribution;
	tagMap["empiricalData"] = eEmpiricalData;

	for (DOMElement * element = XercesUtilities::getFirstChildDOMElement(transformation); element != 0; element
			= XercesUtilities::getNextSiblingDOMElement(element)) {
		XercesString tagName = XMLString::replicate(element->getTagName());
		auto_ptr<MarkDistribution> dist;
		switch (tagMap[tagName.toString()]) {
		case (eNormalDistribution): {
			dist.reset(new MarkNormalDistribution);
			break;
		}
		case (eWeibullDistribution): {
			dist.reset(new MarkWeibullDistribution);
			break;
		}
		case (eEmpiricalCumulativeDistribution): {
			dist.reset(new MarkECDFDistribution);
			break;
		}
		case (eEmpiricalData): {
			dist.reset(new MarkDataDistribution);
			break;
		}
		default: { // Handles eUnrecognized.
			throw "Unrecognized XML-element: "+tagName.toString();
		}
		}
		dist->loadData(element);
		dist->setUp(targetCDFClasses, targetCDFValues);
	}

	loaded = true;
}

