#ifndef MARKTRANSFORMATION_H_
#define MARKTRANSFORMATION_H_

#include <vector>
using std::vector;
#include <string>
using std::string;

#ifdef USE_MPI
#include <mpi.h>
#endif

#include <xercesc/util/XercesVersion.hpp>

namespace XERCES_CPP_NAMESPACE {
class DOMElement;
}

/**
 * Contains a cumulative distribution which can be used by the MarkTransformer
 * class to transform tree marks to the desired distribution. It uses the
 * MarkDistribution classes to generate the actual CDF.
 */
class MarkTransformation {
public:
	MarkTransformation();
	~MarkTransformation();

#ifdef USE_MPI
	void broadcast(int const rootId, MPI_Comm const comm);
#endif

	/**
	 * Load parameters from XML.
	 * @param transformation Should correspond to a <transformation> element
	 */
	void loadData(XERCES_CPP_NAMESPACE::DOMElement const * transformation);

	inline bool isLoaded() const {
		return loaded;
	}
	inline vector<double> const & getCDFClasses() const {
		return targetCDFClasses;
	}
	inline vector<double> const & getCDFValues() const {
		return targetCDFValues;
	}
private:
	bool loaded;
	vector<double> targetCDFClasses;
	vector<double> targetCDFValues;
};

#endif /* MARKTRANSFORMATION_H_ */
