#ifndef SPECIES_H_
#define SPECIES_H_

#include "CompetitionDistanceData.h"

#include <string>
using std::string;
#include <cassert>
#include <vector>
using std::vector;

#ifdef USE_MPI
#include <mpi.h>
#endif

#include <xercesc/util/XercesVersion.hpp>

namespace XERCES_CPP_NAMESPACE {
	class DOMElement;
}

class Species {
public:
	enum SpeciesVariables {
		SECONDARY_MODEL_VALUE = 0,
		SMALL_VOLUME_COEFFICIENT,
		STUMP_MIN_HEIGHT, STUMP_DIAMETER_COEFFICIENT, STUMP_HEIGHT_COEFFICIENT,
		TAPER_COEFFICIENT1, TAPER_COEFFICIENT2, TAPER_COEFFICIENT3,
		TAPER_COEFFICIENT5, TAPER_COEFFICIENT8, TAPER_COEFFICIENT13,
		TAPER_COEFFICIENT21, TAPER_COEFFICIENT34,
		NUMBER_OF_SPECIES_VARIABLES };

	Species();
	~Species();

#ifdef USE_MPI
	void broadcast(int const rootId, MPI_Comm const comm);
#endif

	inline string const & getName() const {
		return name;
	}
	inline double const & get(enum SpeciesVariables const id) const {
		assert(id < NUMBER_OF_SPECIES_VARIABLES);
		return variables[id];
	}

	void loadData(XERCES_CPP_NAMESPACE::DOMElement const * species);

	inline CompetitionDistanceData const & getCompetitionDistanceData() const {
		return competitionDistanceData;
	}
private:
	void loadTaperCurveData(XERCES_CPP_NAMESPACE::DOMElement const * parameters);

	void loadStumpHeightData(XERCES_CPP_NAMESPACE::DOMElement const * parameters);

	string	name;
	vector<double>	variables;
	CompetitionDistanceData competitionDistanceData;
};
#endif /*SPECIES_H_*/
