#ifndef SPECIESCONTAINER_H_
#define SPECIESCONTAINER_H_

#include <vector>
using std::vector;
#include <string>
using std::string;

#ifdef USE_MPI
#include <mpi.h>
#endif

#include <xercesc/util/XercesVersion.hpp>

namespace XERCES_CPP_NAMESPACE {
	class DOMElement;
}

class Species;

class SpeciesContainer {
public:
	static void Destroy();

#ifdef USE_MPI
	static void broadcast(int const rootId, MPI_Comm const comm);
#endif

	static inline size_t size() {
		return speciesList.size();
	}
	static inline Species * get(string const & name) {
		return get(getId(name));
	}
	static inline size_t getLongestNameLength() {
		return longestNameLength;
	}

	static Species * get(size_t const id);
	static void add(Species * const species);
	static unsigned int getId(string const & name);
	static void loadData(XERCES_CPP_NAMESPACE::DOMElement const * species);

private:
	static vector<Species*> speciesList;
	static size_t longestNameLength;

};

#endif /*SPECIESCONTAINER_H_*/
