#ifndef RECTANGLECONTAINER_H_
#define RECTANGLECONTAINER_H_

#include <vector>
using std::vector;

#include "Point.h"

class TreeContainer;
class Tree;

class RectangleContainer {
public:

	RectangleContainer(TreeContainer const & parent);
	virtual ~RectangleContainer();
	/**
	 * Returns a vector containing trees which are relevant for the calculation of competition indexes
	 */
	vector<Tree*> const & getVectorContaining(Tree const * const tree);

	virtual void add(Tree* tree);

	inline virtual void setLimitingValue(double const val) {
		if(val!=limitingValue) {
			setRectanglesOutDated();
			setTreesOutDated();
			limitingValue = val;
		}
	}
	inline virtual void setTreesOutDated() {
		itsTreesByLocationUpToDate = false;
	}
	inline virtual void setRectanglesOutDated() {
		rectanglesUpToDate = false;
	}
	virtual void clear();
	virtual size_t approxSizeInBytes() const;
	virtual size_t trueSizeInBytes() const;

private:
	//A vector of Trees for every x and y.
	vector< vector< vector<Tree*> > > itsTreesByLocation;
	bool itsTreesByLocationUpToDate;
#ifdef DISABLE_RECTANGLES
	vector<Tree*> allTrees;
#endif

protected:
	TreeContainer const & parentTreeContainer;
	Point minPoint, maxPoint;
	double limitingValue;
	bool rectanglesUpToDate;

	void setupRectangles();
	virtual void placeAllTrees();
	vector<Tree*> & getRectangleContainingPoint(double const xCoord, double const yCoord);
	size_t getRectangleCoordinate(double const coord, double const minCoord, double const maxCoord, size_t const numberOfRectangles) const;
	void setBoundingBoxCoordinates();
	virtual void placeTree(Tree * const curTree);

};
#endif /*RECTANGLECONTAINER_H_*/
