/*
 * ForestInitializer.h
 *
 *  Created on: 2008-nov-05
 *      Author: joschori
 */

#ifndef FORESTINITIALIZER_H_
#define FORESTINITIALIZER_H_

#include <string>
using std::string;

#include "TreeMarks.h"

class TreeContainer;
class InitialCompartmentGenerator;

class ForestInitializer {
public:
	// Load data if available, keep track of which are loaded
	static void load(string const & filename, TreeContainer & trees);
	// Generate data that has not been loaded
	static void generate(InitialCompartmentGenerator const & compartmentGenerator, int const replication, TreeContainer & trees);

	inline static bool treesExist() { return treesLoaded; }
	inline static bool positionIsLoaded() { return positionLoaded; }
	inline static bool speciesIsLoaded() { return speciesLoaded; }
	inline static bool heterogeneityIsLoaded() { return heterogeneityLoaded; }
	inline static bool ageIsLoaded() { return ageLoaded; }
	inline static bool markIsLoaded(size_t const index) { return marksLoaded[index]; }
	inline static bool allMarksLoaded() {
		for(size_t index=0; index<TreeMarks::NUMBER_OF_MARKS; index++) {
			if(!marksLoaded[index])
				return false;
		}
		return true;
	}

private:
	static bool treesLoaded;
	static bool positionLoaded;
	static bool speciesLoaded;
	static bool heterogeneityLoaded;
	static bool ageLoaded;
	static bool marksLoaded[TreeMarks::NUMBER_OF_MARKS];

	static void checkFile(string const & filename);
};

#endif /* FORESTINITIALIZER_H_ */
