#ifndef INITIALCOMPARTMENTGENERATOR_H_
#define INITIALCOMPARTMENTGENERATOR_H_

#include <memory>
using std::auto_ptr;
#ifdef USE_MPI
#include <mpi.h>
#endif
#include "InitializationData.h"

#include <xercesc/util/XercesVersion.hpp>

namespace XERCES_CPP_NAMESPACE
{
	class DOMElement;
}

class SimulationData;
class PointGenerator;
class Compartment;
class TreeContainer;

/*
 * Generates an initial compartment for simulation according to the parameters.
 */
class InitialCompartmentGenerator {
public:
	InitialCompartmentGenerator(SimulationData const & sim);
	~InitialCompartmentGenerator();
	void generateTreeData(Compartment * compartment) const;
	void generateTreeMarks(TreeContainer & allTrees) const;

#ifdef USE_MPI
	void broadcast(int const rootId, MPI_Comm const comm);
#endif

#ifdef INCLUDE_ENVIRONMENTAL_EFFECTS
	bool hasEnvironmentalEffects() const;
#endif

	/**
	 * Loads XML data for the InitialCompartmentGenerator.
	 *
	 * Precondition:	@initialCompartmentGenerator must not be null.
	 */
	void loadData(XERCES_CPP_NAMESPACE::DOMElement const * initialCompartmentGenerator);
private:
	/**
	 * Loads XML data for the PointGenerator.
	 *
	 * Precondition:	@pointProcess must not be null.
	 */
	void loadPointProcess(XERCES_CPP_NAMESPACE::DOMElement const * pointProcess);

	SimulationData const &		simulationData;
	auto_ptr<PointGenerator>	pointGenerator;
	vector<InitializationData>	initializationDatas;
	unsigned int				averageInitialAge;
};

#endif /*INITIALCOMPARTMENTGENERATOR_H_*/
