#include "XercesUtilities.h"
#include "XercesString.h"

#include <xercesc/dom/DOMElement.hpp>
#include <xercesc/dom/DOMNode.hpp>
#include <xercesc/util/XMLString.hpp>
XERCES_CPP_NAMESPACE_USE

XercesUtilities::XercesUtilities()
{
}

XercesUtilities::~XercesUtilities()
{
}

DOMElement * XercesUtilities::getFirstChildDOMElement(DOMElement const * parent)
{	
	DOMNode * child = parent->getFirstChild();
	while(child != 0 && child->getNodeType() != DOMNode::ELEMENT_NODE) {
		child = child->getNextSibling();
	}
	return static_cast<DOMElement *>(child);
}

DOMElement * XercesUtilities::getNextSiblingDOMElement(DOMElement const * element)
{
	DOMNode * sibling = element->getNextSibling();
	while(sibling != 0 && sibling->getNodeType() != DOMNode::ELEMENT_NODE) {
		sibling = sibling->getNextSibling();
	}
	return static_cast<DOMElement *>(sibling);
}

int XercesUtilities::getFirstChildTextContentAsInt(DOMElement const * element)
{
	XercesString content = getFirstChildTextContent(element);
	return content.toInt();
}

double XercesUtilities::getFirstChildTextContentAsDouble(DOMElement const * element)
{
	XercesString content = getFirstChildTextContent(element);
	return content.toDouble();
}

string XercesUtilities::getFirstChildTextContentAsString(DOMElement const * element)
{
	XercesString content = getFirstChildTextContent(element);
	return content.toString();
}

int XercesUtilities::getAttributeValueAsInt(DOMElement const * element, string const & attributeName)
{
	XercesString attributeValue = getAttributeValue(element, attributeName);
	return attributeValue.toInt();
}

double XercesUtilities::getAttributeValueAsDouble(DOMElement const * element, string const & attributeName)
{
	XercesString attributeValue = getAttributeValue(element, attributeName);
	return attributeValue.toDouble();
}

string XercesUtilities::getAttributeValueAsString(DOMElement const * element, string const & attributeName)
{
	XercesString attributeValue = getAttributeValue(element, attributeName);
	return attributeValue.toString();
}

XercesString XercesUtilities::getFirstChildTextContent(DOMElement const * element)
{
	DOMNode * child = element->getFirstChild();
	if(child != NULL && child->getNodeType() == DOMNode::TEXT_NODE) {
		return XMLString::replicate(child->getNodeValue());
	} else {
		string str = "Text node expected as child of ";
		str.append(XMLString::transcode(element->getTagName()));
		throw str;
	}
}

XercesString XercesUtilities::getAttributeValue(DOMElement const * element, string const & attributeName)
{
	XercesString name = XMLString::transcode(attributeName.c_str());
	return XMLString::replicate(element->getAttribute(name.xmlCh()));
}
