#ifndef XERCESUTILITIES_H_
#define XERCESUTILITIES_H_

#include <string>
using std::string;

#include <xercesc/util/XercesVersion.hpp>

namespace XERCES_CPP_NAMESPACE
{
	class DOMElement;
}

class XercesString;

/**
 * Convenience class for extracting information from Xerces' DOM tree.
 */
class XercesUtilities
{
public:
	XercesUtilities();
	~XercesUtilities();
	
	/**
	 * Returns @parent's first child node that is a DOMNode::ELEMENT_NODE or null if
	 * no such node can be found.
	 * 
	 * Precondition:	@parent must not be null.
	 */
	static XERCES_CPP_NAMESPACE::DOMElement * getFirstChildDOMElement(XERCES_CPP_NAMESPACE::DOMElement const * parent);
	/**
	 * Returns @element's next sibling node that is a DOMNode::ELEMENT_NODE or null if
	 * no such node can be found.
	 * 
	 * Precondition:	@element must not be null.
	 */
	static XERCES_CPP_NAMESPACE::DOMElement * getNextSiblingDOMElement(XERCES_CPP_NAMESPACE::DOMElement const * element);
	/**
	 * Returns the content of @element's first child node as an int. This node
	 * must be a DOMNode::TEXT_NODE.
	 * 
	 * Precondition:	@element must be a pointer to a valid DOMElement, i.e. it must not be null
	 * 					and element->getFirstChild() must not return null.
	 */
	static int getFirstChildTextContentAsInt(XERCES_CPP_NAMESPACE::DOMElement const * element);
	/**
	 * Returns the content of @element's first child node as a double. This node
	 * must be a DOMNode::TEXT_NODE.
	 * 
	 * Precondition:	@element must be a pointer to a valid DOMElement, i.e. it must not be null
	 * 					and element->getFirstChild() must not return null.
	 */
	static double getFirstChildTextContentAsDouble(XERCES_CPP_NAMESPACE::DOMElement const * element);
	/**
	 * Returns the content of @element's first child node as a string. This node
	 * must be a DOMNode::TEXT_NODE.
	 * 
	 * Precondition:	@element must be a pointer to a valid DOMElement, i.e. it must not be null
	 * 					and element->getFirstChild() must not return null.
	 */
	static string getFirstChildTextContentAsString(XERCES_CPP_NAMESPACE::DOMElement const * element);
	/**
	 * Returns the value of the attribute identified by @attributeName as an int.
	 * 
	 * Precondition:	@element must not be null
	 */
	static int getAttributeValueAsInt(XERCES_CPP_NAMESPACE::DOMElement const * element, string const & attributeName);
	/**
	 * Returns the value of the attribute identified by @attributeName as a double.
	 * 
	 * Precondition:	@element must not be null
	 */
	static double getAttributeValueAsDouble(XERCES_CPP_NAMESPACE::DOMElement const * element, string const & attributeName);
	/**
	 * Returns the value of the attribute identified by @attributeName as a string.
	 * 
	 * Precondition:	@element must not be null
	 */
	static string getAttributeValueAsString(XERCES_CPP_NAMESPACE::DOMElement const * element, string const & attributeName);
private:
	/**
	 * Returns the content of @element's first child node in a XercesString. This node
	 * must be a DOMNode::TEXT_NODE.
	 * 
	 * Precondition:	@element must be a pointer to a valid DOMElement, i.e. it must not be null
	 * 					and element->getFirstChild() must not return null.
	 */
	 static XercesString getFirstChildTextContent(XERCES_CPP_NAMESPACE::DOMElement const * element);
	/**
	 * Returns the value of the attribute identified by @attributeName in a XercesString.
	 * 
	 * Precondition:	@element must not be null
	 */
	static XercesString getAttributeValue(XERCES_CPP_NAMESPACE::DOMElement const * element, string const & attributeName);
};

#endif /*XERCESUTILITIES_H_*/
