/*
 * ModelTest.cpp
 *
 *  Created on: 22.4.2009
 *      Author: asignell
 */
#include "ModelTest.h"
#include "math.h"
// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( ModelTest );
ModelTest::ModelTest() {

}

void ModelTest::addTree(Tree* tree) {
	tree->setMark(TreeMarks::HEIGHT, tree->getMark(TreeMarks::DIAMETER)*2.0);
	trees.push_back(tree);

}

void ModelTest::setUp() {
	trees.clear();
	addTree(new Tree(0, 0, Point(4.0, 2.0), 0.45));
	addTree(new Tree(0, 0, Point(6.0, 4.0), 0.3));
	addTree(new Tree(0, 0, Point(0.0, 0.0), 1.0));
	addTree(new Tree(0, 0, Point(5.0, 0.0), 0.2));
	addTree(new Tree(0, 0, Point(1.0, 6.0), 0.10));
	addTree(new Tree(0, 0, Point(15.0, 10.0), 1.0));
	addTree(new Tree(0, 0, Point(10.0, 5.0), .80));

}

void ModelTest::tearDown() {
}

void ModelTest::testModel() {
	Vector modelVariables(Model::NUMBER_OF_MODEL_VARIABLES);

	Tree * targetTree = trees[0];

	Model::calculateModelVariables(targetTree, trees, 5.0f, modelVariables);

	// These are not set by calculateModelVariables and should be zero
	CPPUNIT_ASSERT_EQUAL(0.0, modelVariables[Model::TAU]);
	CPPUNIT_ASSERT_EQUAL(0.0, modelVariables[Model::XI]);
	CPPUNIT_ASSERT_EQUAL(0.0, modelVariables[Model::EPSILON]);

	float diameter = 0.45f;
	float height = 0.9f;
	double logdiam = log(diameter);
	double density = 4.0/(5.0*5.0*M_PI) * 10000.0;
	double avgDist = (sqrt(5.0)+sqrt(20.0)+sqrt(8.0)+sqrt(25.0))/4.0;
	double invDist = 1/sqrt(20.0);
	double competitorDiamSum = (double)(float)0.3f+1.0f+0.2f+0.1f;
	double diamDiff = (diameter-1.0f)+(diameter-0.2f)+(diameter-0.3f)+(diameter-0.1f);
	double heightDiff = (height-2.0f)+(height-0.4f)+(height-0.6f)+(height-0.2f);
	double diamRatio = (double)(float)1.0f/0.45f;
	double heightDiameterRatio = height/diameter;
	double competitorBasalArea = (M_PI*0.5f*0.5f+M_PI*0.1f*0.1f+M_PI*0.15f*0.15f+M_PI*0.05f*0.05f) / 10000.0;
	double relBasalArea = (M_PI*(diameter/200.0)*(diameter/200.0)) / competitorBasalArea;
	double basalAreaDiamRatio = competitorBasalArea / (competitorDiamSum / 4.0f);
	competitorBasalArea = competitorBasalArea / (5.0*5.0*M_PI) * 10000.0;

	double const delta = 0.000001;
	// These are all set by calculateModelVariables
	CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, modelVariables[Model::INTERCEPT], delta);
	CPPUNIT_ASSERT_DOUBLES_EQUAL(diameter, (float) modelVariables[Model::DIAMETER], delta);
	CPPUNIT_ASSERT_DOUBLES_EQUAL(logdiam, modelVariables[Model::DIAMETER_LOG], delta);
	CPPUNIT_ASSERT_DOUBLES_EQUAL(density, modelVariables[Model::DENSITY], delta);
	CPPUNIT_ASSERT_DOUBLES_EQUAL(avgDist, modelVariables[Model::AVG_DIST], delta);
	CPPUNIT_ASSERT_DOUBLES_EQUAL(invDist, modelVariables[Model::INV_DIST], delta);
	CPPUNIT_ASSERT_DOUBLES_EQUAL(competitorDiamSum, modelVariables[Model::DIAMETER_SUM], delta);
	CPPUNIT_ASSERT_DOUBLES_EQUAL(diamDiff, modelVariables[Model::DIAMETER_DIFF], delta);
	CPPUNIT_ASSERT_DOUBLES_EQUAL(diamRatio, modelVariables[Model::DIAMETER_RATIO], delta);
	CPPUNIT_ASSERT_DOUBLES_EQUAL(heightDiff, modelVariables[Model::HEIGHT_DIFF], delta);
	CPPUNIT_ASSERT_DOUBLES_EQUAL(heightDiameterRatio, modelVariables[Model::HEIGHT_DIAMETER_RATIO], delta);
	CPPUNIT_ASSERT_DOUBLES_EQUAL(competitorBasalArea, modelVariables[Model::COMPETITOR_BASAL_AREA], delta);
	CPPUNIT_ASSERT_DOUBLES_EQUAL(relBasalArea, modelVariables[Model::RELATIVE_BASAL_AREA], delta);
	CPPUNIT_ASSERT_DOUBLES_EQUAL(basalAreaDiamRatio, modelVariables[Model::BA_DIAMETER_RATIO], delta);
}

