#include "Vector.h"

Vector::Vector()
: vec(NULL)
, vecsize(0)
, ownsData(true)
{
}

Vector::Vector(Vector const & other)
: vec(new double[other.vecsize])
, vecsize(other.vecsize)
, ownsData(true)
{
	copyVector(other,*this);
}

Vector::Vector(vector<double> const & other)
: vec(new double[other.size()])
, vecsize(other.size())
, ownsData(true)
{
	Vector dummy = Vector((double*)&other[0],other.size()); // Exploiting the fact that a vector is really an array.
	copyVector(dummy,*this);
}

Vector::Vector(vector<int> const & other)
: vec(new double[other.size()])
, vecsize(other.size())
, ownsData(true)
{
	for(size_t index=0; index<vecsize; index++) {
		vec[index] = other[index];
	}
}

Vector::Vector(double * row, size_t numberOfElements)
: vec(row)
, vecsize(numberOfElements)
, ownsData(false)
{

}

Vector::Vector(size_t size, double defaultValue)
: vec(new double[size])
, vecsize(size)
, ownsData(true)
{
	for(size_t i=0; i<vecsize; i++) {
		vec[i] = defaultValue;
	}
}

Vector::~Vector()
{
	if(ownsData) delete[] vec;
}

Vector operator+(Vector const & left, Vector const & right) {
	Vector temp(left);
	temp += right;
	return temp;
}

ostream & operator<<(ostream & os, Vector const & v) {
	const size_t size = v.size();
	for(size_t i=0; i<size; ++i) {
		os << v[i] << " ";
	}
	return os;
}
